@@ -37,6 +37,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <stdint.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netdb.h>
@@ -44,6 +45,7 @@
 #include <OpenIPMI/ipmi_cmdlang.h>
 #include <OpenIPMI/ipmi_pet.h>
 #include <OpenIPMI/ipmi_lanparm.h>
+#include <OpenIPMI/ipmi_solparm.h>
 #include <OpenIPMI/ipmi_fru.h>
 #include <OpenIPMI/ipmi_pef.h>
 #include <OpenIPMI/ipmi_auth.h>
@@ -88,8 +90,9 @@ struct ipmi_cmd_info_s
 
 struct ipmi_cmdlang_cmd_s
 {
-    char               *name;
-    char               *help;
+    char                  *name;
+    char                  *help;
+    ipmi_help_finisher_cb help_finish;
 
     /* Only one of handler or subcmds may be non-NULL. */
     ipmi_cmdlang_handler_cb handler;
@@ -431,6 +434,103 @@ ipmi_cmdlang_lanparm_handler(ipmi_cmd_info_t *cmd_info)
 		 cmd_info);
 }
 
+/*
+ * Handling for iterating SOLPARMs.
+ */
+typedef struct solparm_iter_info_s
+{
+    char                *cmdstr;
+    ipmi_solparm_ptr_cb handler;
+    void                *cb_data;
+    ipmi_cmd_info_t     *cmd_info;
+} solparm_iter_info_t;
+
+static void
+for_each_solparm_handler(ipmi_solparm_t *solparm, void *cb_data)
+{
+    solparm_iter_info_t *info = cb_data;
+    ipmi_cmd_info_t *cmd_info = info->cmd_info;
+    ipmi_cmdlang_t  *cmdlang = ipmi_cmdinfo_get_cmdlang(cmd_info);
+    char            name[IPMI_SOLPARM_NAME_LEN];
+    char            *c;
+
+    if (cmdlang->err)
+	return;
+
+    ipmi_solparm_get_name(solparm, name, sizeof(name));
+
+    c = strrchr(name, '.');
+    if (!c)
+	goto out_err;
+    c++;
+    if ((! info->cmdstr) || (strcmp(info->cmdstr, c) == 0))
+	info->handler(solparm, info->cb_data);
+    return;
+
+ out_err:
+    ipmi_cmdlang_global_err(name,
+			    "cmdlang.c(for_each_solparm_handler)",
+			    "Bad SOLPARM name", EINVAL);
+}
+
+static void
+for_each_solparm_domain_handler(ipmi_domain_t *domain, void *cb_data)
+{
+    ipmi_solparm_iterate_solparms(domain, for_each_solparm_handler, cb_data);
+}
+
+static void
+for_each_solparm(ipmi_cmd_info_t *cmd_info,
+	     char            *domain,
+	     char            *class,
+	     char            *obj,
+	     ipmi_solparm_ptr_cb handler,
+	     void            *cb_data)
+{
+    solparm_iter_info_t info;
+
+    if (class) {
+	cmd_info->cmdlang->errstr = "Invalid SOLPARM";
+	cmd_info->cmdlang->err = EINVAL;
+	cmd_info->cmdlang->location = "cmdlang.c(for_each_solparm)";
+	return;
+    }
+
+    info.handler = handler;
+    info.cb_data = cb_data;
+    info.cmd_info = cmd_info;
+    info.cmdstr = obj;
+    for_each_domain(cmd_info, domain, NULL, NULL,
+		    for_each_solparm_domain_handler, &info);
+}
+
+void
+ipmi_cmdlang_solparm_handler(ipmi_cmd_info_t *cmd_info)
+{
+    char *domain, *class, *obj;
+
+    if (cmd_info->curr_arg >= cmd_info->argc) {
+	domain = class = obj = NULL;
+    } else {
+	domain = cmd_info->argv[cmd_info->curr_arg];
+	class = NULL;
+	obj = strrchr(domain, '.');
+	if (!obj) {
+	    cmd_info->cmdlang->errstr = "Invalid SOLPARM";
+	    cmd_info->cmdlang->err = EINVAL;
+	    cmd_info->cmdlang->location
+		= "cmdlang.c(ipmi_cmdlang_solparm_handler)";
+	    return;
+	}
+	*obj = '\0';
+	obj++;
+	cmd_info->curr_arg++;
+    }
+
+    for_each_solparm(cmd_info, domain, class, obj, cmd_info->handler_data,
+		 cmd_info);
+}
+
 /*
  * Handling for iterating PEFs.
  */
@@ -1236,9 +1336,11 @@ ipmi_cmdlang_handle(ipmi_cmdlang_t *cmdlang, char *str)
 	next_help:
 	    if (argc == curr_arg) {
 		rv = 0;
-		if (parent)
+		if (parent) {
 		    cmdlang->out(cmdlang, parent->name, parent->help);
-		else
+		    if (parent->help_finish)
+			parent->help_finish(cmdlang);
+		}else
 		    cmdlang->out(cmdlang, "help", NULL);
 		if (cmdlang->err)
 		    goto done_help;
@@ -1249,6 +1351,8 @@ ipmi_cmdlang_handle(ipmi_cmdlang_t *cmdlang, char *str)
 			cmdlang->up(cmdlang);
 			goto done_help;
 		    }
+		    if (cmd->help_finish)
+			cmd->help_finish(cmdlang);
 		    cmd = cmd->next;
 		}
 		cmdlang->up(cmdlang);
@@ -1330,6 +1434,7 @@ ipmi_cmdlang_reg_cmd(ipmi_cmdlang_cmd_t      *parent,
 		     char                    *help,
 		     ipmi_cmdlang_handler_cb handler,
 		     void                    *cb_data,
+		     ipmi_help_finisher_cb   help_finish,
 		     ipmi_cmdlang_cmd_t      **new_val)
 {
     ipmi_cmdlang_cmd_t *rv;
@@ -1355,6 +1460,7 @@ ipmi_cmdlang_reg_cmd(ipmi_cmdlang_cmd_t      *parent,
     rv->handler = handler;
     rv->subcmds = NULL;
     rv->handler_data = cb_data;
+    rv->help_finish = help_finish;
     rv->next = NULL;
 
     if (parent) {
@@ -1396,6 +1502,7 @@ ipmi_cmdlang_reg_table(ipmi_cmdlang_init_t *table, int len)
 				  table[i].help,
 				  table[i].handler,
 				  table[i].cb_data,
+				  table[i].help_finish,
 				  table[i].new_val);
 	if (rv)
 	    return rv;
@@ -1475,6 +1582,7 @@ ipmi_cmdlang_out_binary(ipmi_cmd_info_t *info,
 			const char      *value,
 			unsigned int    len)
 {
+    info->did_output = 1;
     info->cmdlang->out_binary(info->cmdlang, name, value, len);
 }
 
@@ -1484,6 +1592,7 @@ ipmi_cmdlang_out_unicode(ipmi_cmd_info_t *info,
 			 const char      *value,
 			 unsigned int    len)
 {
+    info->did_output = 1;
     info->cmdlang->out_unicode(info->cmdlang, name, value, len);
 }
 
@@ -1546,7 +1655,7 @@ ipmi_cmdlang_out_ip(ipmi_cmd_info_t *info,
 		    struct in_addr  *ip_addr)
 {
     char outstr[16];
-    u_int32_t addr = ntohl(ip_addr->s_addr);
+    uint32_t addr = ntohl(ip_addr->s_addr);
 
     /* Why isn't there an inet_ntoa_r? */
     sprintf(outstr, "%d.%d.%d.%d",
@@ -2617,8 +2726,10 @@ int ipmi_cmdlang_entity_init(os_handler_t *os_hnd);
 int ipmi_cmdlang_mc_init(os_handler_t *os_hnd);
 int ipmi_cmdlang_pet_init(os_handler_t *os_hnd);
 int ipmi_cmdlang_lanparm_init(os_handler_t *os_hnd);
+int ipmi_cmdlang_solparm_init(os_handler_t *os_hnd);
 int ipmi_cmdlang_fru_init(os_handler_t *os_hnd);
 void ipmi_cmdlang_lanparm_shutdown();
+void ipmi_cmdlang_solparm_shutdown();
 int ipmi_cmdlang_pef_init(os_handler_t *os_hnd);
 void ipmi_cmdlang_pef_shutdown();
 int ipmi_cmdlang_sensor_init(os_handler_t *os_hnd);
@@ -2648,6 +2759,9 @@ ipmi_cmdlang_init(os_handler_t *os_hnd)
     rv = ipmi_cmdlang_lanparm_init(os_hnd);
     if (rv) return rv;
 
+    rv = ipmi_cmdlang_solparm_init(os_hnd);
+    if (rv) return rv;
+
     rv = ipmi_cmdlang_fru_init(os_hnd);
     if (rv) return rv;
 
@@ -2688,6 +2802,7 @@ ipmi_cmdlang_cleanup(void)
 {
     ipmi_cmdlang_pef_shutdown();
     ipmi_cmdlang_lanparm_shutdown();
+    ipmi_cmdlang_solparm_shutdown();
     cleanup_level(cmd_list);
 }
 
