@@ -31,8 +31,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef _IPMI_SENSOR_H
-#define _IPMI_SENSOR_H
+#ifndef OPENIPMI_SENSOR_H
+#define OPENIPMI_SENSOR_H
 
 #include <OpenIPMI/ipmi_types.h>
 #include <OpenIPMI/ipmi_addr.h>
@@ -48,11 +48,11 @@ int ipmi_sensors_alloc(ipmi_mc_t *mc, ipmi_sensor_info_t **new_sensors);
 /* Destroy a sensor repository and all the sensors in it. */
 int ipmi_sensors_destroy(ipmi_sensor_info_t *sensors);
 
-/* Must be called with the _ipmi_domain_entity_lock() held. */
-int _ipmi_sensor_get(ipmi_sensor_t *sensor);
+/* Must be called with the i_ipmi_domain_entity_lock() held. */
+int i_ipmi_sensor_get(ipmi_sensor_t *sensor);
 
 /* Must be called with no locks held. */
-void _ipmi_sensor_put(ipmi_sensor_t *sensor);
+void i_ipmi_sensor_put(ipmi_sensor_t *sensor);
 
 /* Return the number of sensors in the data structure. */
 unsigned int ipmi_sensors_get_count(ipmi_sensor_info_t *sensors);
@@ -509,4 +509,4 @@ int ipmi_sensor_pointer_noseq_cb(ipmi_sensor_id_t   id,
 				 ipmi_sensor_ptr_cb handler,
 				 void               *cb_data);
 
-#endif /* _IPMI_SENSOR_H */
+#endif /* OPENIPMI_SENSOR_H */
