@@ -31,10 +31,13 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef __IPMI_CMDLANG_H
-#define __IPMI_CMDLANG_H
+#ifndef OPENIPMI_CMDLANG_H
+#define OPENIPMI_CMDLANG_H
 
 #include <OpenIPMI/selector.h>
+#include <OpenIPMI/ipmi_bits.h>
+#include <OpenIPMI/ipmi_types.h>
+#include <OpenIPMI/ipmi_addr.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -127,6 +130,7 @@ int ipmi_cmdlang_get_evinfo(void);
 typedef struct ipmi_cmdlang_cmd_s ipmi_cmdlang_cmd_t;
 
 typedef void (*ipmi_cmdlang_handler_cb)(ipmi_cmd_info_t *cmd_info);
+typedef void (*ipmi_help_finisher_cb)(ipmi_cmdlang_t *cmdlang);
 
 /* Register a command as a subcommand of the parent, or into the main
    command list if parent is NULL.  The command will have the given
@@ -140,6 +144,7 @@ int ipmi_cmdlang_reg_cmd(ipmi_cmdlang_cmd_t      *parent,
 			 char                    *help,
 			 ipmi_cmdlang_handler_cb handler,
 			 void                    *handler_data,
+			 ipmi_help_finisher_cb   help_finish,
 			 ipmi_cmdlang_cmd_t      **rv);
 
 /* Register a table of commands. */
@@ -151,6 +156,7 @@ typedef struct ipmi_cmdlang_init_s
     ipmi_cmdlang_handler_cb handler;
     void                    *cb_data;
     ipmi_cmdlang_cmd_t      **new_val;
+    ipmi_help_finisher_cb   help_finish;
 } ipmi_cmdlang_init_t;
 int ipmi_cmdlang_reg_table(ipmi_cmdlang_init_t *table, int len);
 
@@ -198,6 +204,9 @@ void ipmi_cmdlang_pet_handler(ipmi_cmd_info_t *cmd_info);
 /* ipmi_lanparm_ptr_cb */
 void ipmi_cmdlang_lanparm_handler(ipmi_cmd_info_t *cmd_info);
 
+/* ipmi_solparm_ptr_cb */
+void ipmi_cmdlang_solparm_handler(ipmi_cmd_info_t *cmd_info);
+
 /* ipmi_fru_ptr_cb */
 void ipmi_cmdlang_fru_handler(ipmi_cmd_info_t *cmd_info);
 
@@ -356,4 +365,4 @@ ipmi_cmdlang_t *ipmi_cmdinfo_get_cmdlang(ipmi_cmd_info_t *info);
 }
 #endif
 
-#endif /* __IPMI_CMDLANG_H */
+#endif /* OPENIPMI_CMDLANG_H */
