@@ -32,8 +32,8 @@
  *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ifndef _IPMI_CONN_H
-#define _IPMI_CONN_H
+#ifndef OPENIPMI_CONN_H
+#define OPENIPMI_CONN_H
 
 #include <OpenIPMI/ipmi_types.h>
 #include <OpenIPMI/ipmi_addr.h>
@@ -414,13 +414,20 @@ struct ipmi_con_s
        not long enough to hold it. */
     int (*get_port_info)(ipmi_con_t *ipmi, unsigned int port,
 			 char *info, int *info_len);
+
+    /* Disable the connection.  This is primarily for handling a fork,
+       make sure nothing happens on the connection after this is
+       called.  To call this, you should make sure that no thread is
+       in any event loop (before the fork) and there are no calls into
+       the library.  After calling this you should call close. */
+    void (*disable)(ipmi_con_t *ipmi);
 };
 
 #define IPMI_CONN_NAME(c) (c->name ? c->name : "")
 
 /* Initialization code for the initialization the connection code. */
-int _ipmi_conn_init(os_handler_t *os_hnd);
-void _ipmi_conn_shutdown(void);
+int i_ipmi_conn_init(os_handler_t *os_hnd);
+void i_ipmi_conn_shutdown(void);
 
 
 /* Address types for external addresses. */
@@ -540,4 +547,4 @@ void ipmi_con_attr_cleanup(ipmi_con_t *con);
 }
 #endif
 
-#endif /* _IPMI_CONN_H */
+#endif /* OPENIPMI_CONN_H */
