@@ -70,21 +70,30 @@
  *	include/OpenIPMI/include/ipmi_sol.h
  */
 
+#ifndef OPENIPMI_SOL_H
+#define OPENIPMI_SOL_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 /**
  * Queue identification constants for ipmi_sol_flush(...).
  */
-#define IPMI_SOL_BMC_TRANSMIT_QUEUE 0x01
-#define IPMI_SOL_BMC_RECEIVE_QUEUE 0x02
+#define IPMI_SOL_BMC_TRANSMIT_QUEUE                0x01
+#define IPMI_SOL_BMC_RECEIVE_QUEUE                 0x02
 #define IPMI_SOL_MANAGEMENT_CONSOLE_TRANSMIT_QUEUE 0x04
-#define IPMI_SOL_MANAGEMENT_CONSOLE_RECEIVE_QUEUE 0x08
+#define IPMI_SOL_MANAGEMENT_CONSOLE_RECEIVE_QUEUE  0x08
+
+#define IPMI_SOL_BMC_QUEUES (IPMI_SOL_BMC_TRANSMIT_QUEUE \
+			     | IPMI_SOL_BMC_RECEIVE_QUEUE)
 
-#define IPMI_SOL_BMC_QUEUES \
-	(IPMI_SOL_BMC_TRANSMIT_QUEUE | IPMI_SOL_BMC_RECEIVE_QUEUE)
 #define IPMI_SOL_MANAGEMENT_CONSOLE_QUEUES \
-	(IPMI_SOL_MANAGEMENT_CONSOLE_TRANSMIT_QUEUE | IPMI_SOL_MANAGEMENT_CONSOLE_RECEIVE_QUEUE)
-#define IPMI_SOL_ALL_QUEUES \
-	(IPMI_SOL_BMC_QUEUES | IPMI_SOL_MANAGEMENT_CONSOLE_QUEUES)
+	(IPMI_SOL_MANAGEMENT_CONSOLE_TRANSMIT_QUEUE \
+	 | IPMI_SOL_MANAGEMENT_CONSOLE_RECEIVE_QUEUE)
+
+#define IPMI_SOL_ALL_QUEUES (IPMI_SOL_BMC_QUEUES \
+			     | IPMI_SOL_MANAGEMENT_CONSOLE_QUEUES)
 
 
 /**
@@ -101,15 +110,36 @@
 /**
  * The possible states of a SoL connection object.
  */
-typedef enum { ipmi_sol_state_closed, ipmi_sol_state_connecting, ipmi_sol_state_connected,
-	ipmi_sol_state_connected_ctu, ipmi_sol_state_closing } ipmi_sol_state;
+typedef enum
+{
+    /* The connection is closed; no data transfer is possible. */
+    ipmi_sol_state_closed,
+
+    /* The connection is currently starting up; no data transfer yet. */
+    ipmi_sol_state_connecting,
+
+    /* The connection is up and operational. */
+    ipmi_sol_state_connected,
+
+    /* The connection is up, but the BMC has reported Character
+       Transfer Unavailable.  This means the BMC is flow-controlling
+       us. */
+    ipmi_sol_state_connected_ctu,
+
+    /* The connection is going down.  No data transfer. */
+    ipmi_sol_state_closing
+} ipmi_sol_state;
 
 
 /**
  * Values to specify serial alert behavior while SoL is activated.
  */
-typedef enum {ipmi_sol_serial_alerts_fail=0, ipmi_sol_serial_alerts_deferred=1,
-	ipmi_sol_serial_alerts_succeed=2} ipmi_sol_serial_alert_behavior;
+typedef enum
+{
+    ipmi_sol_serial_alerts_fail = 0,
+    ipmi_sol_serial_alerts_deferred = 1,
+    ipmi_sol_serial_alerts_succeed = 2
+} ipmi_sol_serial_alert_behavior;
 
 
 /**
@@ -146,11 +176,10 @@ typedef struct ipmi_sol_conn_s ipmi_sol_conn_t;
  * @param [in] cb_data	The user-defined value provided when registering
  *			for the callback.
  */
-typedef void (*ipmi_sol_connection_state_cb)(
-	ipmi_sol_conn_t *conn,
-	int state,
-	int error,
-	void *cb_data);
+typedef void (*ipmi_sol_connection_state_cb)(ipmi_sol_conn_t *conn,
+					     ipmi_sol_state  state,
+					     int             error,
+					     void            *cb_data);
 
 
 /**
@@ -174,20 +203,22 @@ typedef void (*ipmi_sol_connection_state_cb)(
  * @param [in] cb_data	The user-defined value provided when registering
  *			for the callback.
  */
-typedef void (*ipmi_sol_transmit_complete_cb)(
-	ipmi_sol_conn_t *conn,
-	int error,
-	void *cb_data);
+typedef void (*ipmi_sol_transmit_complete_cb)(ipmi_sol_conn_t *conn,
+					      int             error,
+					      void            *cb_data);
 
 
 /**
  * This callback is called asynchronously when characters have been received
  * from the remote.
  *
- * This callback is registered using ipmi_register_data_received_callback.  The
- * recipient of this callback has the opportunity to refuse (NACK) the data by
- * returning a nonzero value; conversely, it should return zero if the packet
- * contents have been accepted.
+ * This callback is registered using
+ * ipmi_register_data_received_callback.  The recipient of this
+ * callback has the opportunity to refuse (NACK) the data by returning
+ * a nonzero value; conversely, it should return zero if the packet
+ * contents have been accepted.  Note that if a NACK is returned, the
+ * BMC will hold packets until ipmi_sol_release_nack() is called or
+ * a flush is done.
  *
  * @param [in] conn	The IPMI SoL connection.
  * @param [in] buf	A pointer to the character data.
@@ -197,11 +228,10 @@ typedef void (*ipmi_sol_transmit_complete_cb)(
  * @return	Zero if the data is accepted, nonzero if the data should be
  *		NACKed.
  */
-typedef int (*ipmi_sol_data_received_cb)(
-	ipmi_sol_conn_t *conn,
-	const void *buf,
-	size_t count,
-	void *cb_data);
+typedef int (*ipmi_sol_data_received_cb)(ipmi_sol_conn_t *conn,
+					 const void      *buf,
+					 size_t          count,
+					 void            *cb_data);
 
 
 /**
@@ -224,11 +254,10 @@ typedef int (*ipmi_sol_data_received_cb)(
  * @param [in] cb_data	The user-defined value provided when registering
  *			for the callback.
  */
-typedef void (*ipmi_sol_flush_complete_cb)(
-	ipmi_sol_conn_t *conn,
-	int error,
-	int queue_selectors_flushed,
-	void *cb_data);
+typedef void (*ipmi_sol_flush_complete_cb)(ipmi_sol_conn_t *conn,
+					   int             error,
+					   int         queue_selectors_flushed,
+					   void            *cb_data);
 
 
 /**
@@ -241,9 +270,8 @@ typedef void (*ipmi_sol_flush_complete_cb)(
  * @param [in] cb_data	The user-defined value provided when registering
  *			for the callback.
  */
-typedef void (*ipmi_sol_break_detected_cb)(
-	ipmi_sol_conn_t *conn,
-	void *cb_data);
+typedef void (*ipmi_sol_break_detected_cb)(ipmi_sol_conn_t *conn,
+					   void            *cb_data);
 
 
 /**
@@ -257,10 +285,8 @@ typedef void (*ipmi_sol_break_detected_cb)(
  * @param [in] cb_data	The user-defined value provided when registering
  *			for the callback.
  */
-typedef void (*ipmi_sol_bmc_transmit_overrun_cb)(
-	ipmi_sol_conn_t *conn,
-	void *cb_data);
-
+typedef void (*ipmi_sol_bmc_transmit_overrun_cb)(ipmi_sol_conn_t *conn,
+						 void            *cb_data);
 
 
 /**
@@ -273,10 +299,8 @@ typedef void (*ipmi_sol_bmc_transmit_overrun_cb)(
  *				IPMI SoL connection structure.
  * @return	zero on success, or ENOMEM if memory allocation fails.
  */
-int ipmi_sol_create(
-	ipmi_con_t *ipmi,
-	ipmi_sol_conn_t **sol_conn
-);
+int ipmi_sol_create(ipmi_con_t      *ipmi,
+		    ipmi_sol_conn_t **sol_conn);
 
 
 
@@ -295,27 +319,32 @@ int ipmi_sol_create(
  * @return Zero on success, or a nonzero error code.
  */
 int ipmi_sol_register_connection_state_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_connection_state_cb cb, void *cb_data);
+					       ipmi_sol_connection_state_cb cb,
+						void            *cb_data);
 int ipmi_sol_deregister_connection_state_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_connection_state_cb cb, void *cb_data);
+					       ipmi_sol_connection_state_cb cb,
+						  void *cb_data);
 
 int ipmi_sol_register_data_received_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_data_received_cb cb, void *cb_data);
+					     ipmi_sol_data_received_cb cb,
+					     void            *cb_data);
 int ipmi_sol_deregister_data_received_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_data_received_cb cb, void *cb_data);
+					       ipmi_sol_data_received_cb cb,
+					       void            *cb_data);
 
 int ipmi_sol_register_break_detected_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_break_detected_cb cb, void *cb_data);
+					      ipmi_sol_break_detected_cb cb,
+					      void            *cb_data);
 int ipmi_sol_deregister_break_detected_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_break_detected_cb cb, void *cb_data);
+						ipmi_sol_break_detected_cb cb,
+						void            *cb_data);
 
 int ipmi_sol_register_bmc_transmit_overrun_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_bmc_transmit_overrun_cb cb, void *cb_data);
+					   ipmi_sol_bmc_transmit_overrun_cb cb,
+						    void            *cb_data);
 int ipmi_sol_deregister_bmc_transmit_overrun_callback(ipmi_sol_conn_t *conn,
-	ipmi_sol_bmc_transmit_overrun_cb cb, void *cb_data);
-
-
-
+					   ipmi_sol_bmc_transmit_overrun_cb cb,
+						      void           *cb_data);
 
 
 /**
@@ -327,6 +356,16 @@ int ipmi_sol_deregister_bmc_transmit_overrun_callback(ipmi_sol_conn_t *conn,
  */
 void ipmi_sol_set_ACK_timeout(ipmi_sol_conn_t *conn, int timeout_usec);
 
+/**
+ * Get the timeout to wait for an ACK from the BMC (for packets that expect an
+ * ACK).
+ *
+ * @param [in] conn	The IPMI SoL connection to configure.
+ *
+ * @return	The timeout
+ */
+int ipmi_sol_get_ACK_timeout(ipmi_sol_conn_t *conn);
+
 
 /**
  * Set the number of retries that we make before declaring a packet "lost".
@@ -336,19 +375,25 @@ void ipmi_sol_set_ACK_timeout(ipmi_sol_conn_t *conn, int timeout_usec);
  */
 void ipmi_sol_set_ACK_retries(ipmi_sol_conn_t *conn, int retries);
 
+/**
+ * Get the number of retries that we make before declaring a packet "lost".
+ *
+ * @param [in] conn	The IPMI SoL connection to configure.
+ *
+ * @return	The number of retries
+ */
+int ipmi_sol_get_ACK_retries(ipmi_sol_conn_t *conn);
 
 /**
  * Configure the authentication to use for the SoL packets.
  *
- * CURRENTLY NOT IMPLEMENTED.
- * 
  * @param [in] conn	The IPMI SoL connection to configure.
  * @param [in] use_authentication	Nonzero to use authentication for the
  * 					SoL packets.
  * @return	Zero on success, otherwise an error code.
  */
 int ipmi_sol_set_use_authentication(ipmi_sol_conn_t *conn,
-	int use_authentication);
+				    int             use_authentication);
 
 /**
  * Query the authentication configuration for the SoL packets.
@@ -363,8 +408,6 @@ int ipmi_sol_get_use_authentication(ipmi_sol_conn_t *conn);
 /**
  * Configure the encryption to use for the SoL packets.
  * 
- * CURRENTLY NOT IMPLEMENTED.
- * 
  * @param [in] conn	The IPMI SoL connection to configure.
  * @param [in] use_encryption	Nonzero to use encryption for the
  *				SoL packets.
@@ -391,7 +434,7 @@ int ipmi_sol_get_use_encryption(ipmi_sol_conn_t *conn);
  * @return	Zero on success, otherwise an error code.
  */
 int ipmi_sol_set_shared_serial_alert_behavior(ipmi_sol_conn_t *conn,
-	ipmi_sol_serial_alert_behavior behavior);
+				      ipmi_sol_serial_alert_behavior behavior);
 
 /**
  * Query the shared serial alerts behavior configuration for the SoL connection.
@@ -400,7 +443,8 @@ int ipmi_sol_set_shared_serial_alert_behavior(ipmi_sol_conn_t *conn,
  * @return	A value identifying the shared serial alert behavior that
  *		will be in force during the connection.
  */
-ipmi_sol_serial_alert_behavior ipmi_sol_get_shared_serial_alert_behavior(ipmi_sol_conn_t *conn);
+ipmi_sol_serial_alert_behavior ipmi_sol_get_shared_serial_alert_behavior
+						      (ipmi_sol_conn_t *conn);
 
 
 /**
@@ -413,7 +457,7 @@ ipmi_sol_serial_alert_behavior ipmi_sol_get_shared_serial_alert_behavior(ipmi_so
  * @return	Zero on success, otherwise an error code.
  */
 int ipmi_sol_set_deassert_CTS_DCD_DSR_on_connect(ipmi_sol_conn_t *conn,
-	int assert);
+						 int             assert);
 
 /**
  * Query the CTS, DCD and DSR deassert-on-connect configuration for the 
@@ -475,13 +519,6 @@ int ipmi_sol_open(ipmi_sol_conn_t *conn);
  * initial state.
  * 
  * @param [in] conn	The IPMI SoL connection to close.
- * @param [in] force	Nonzero if the connection should be forced to close.
- *			Forcing the connection to close will be a synchronous
- *			operation.  All outstanding transmits will be cancelled
- *			and the transmit_complete callbacks will be called
- *			indicating that the transmit failed.  The connection_
- *			state callback will also be called before this function
- *			returns.
  * @return	Zero on success.
  *		EINVAL if this handle was already closing or closed.  If the
  * 			handle is closing, consider using ipmi_sol_force_close
@@ -490,6 +527,23 @@ int ipmi_sol_open(ipmi_sol_conn_t *conn);
  */
 int ipmi_sol_close(ipmi_sol_conn_t *conn);
 
+/**
+ * Requests the closure of the SoL session.
+ *
+ * Closing the SoL session also restores the baseboard serial mux to its
+ * initial state.
+ *
+ * @param [in] conn	The IPMI SoL connection to close.
+ * @param [in] rem_close Nonzero if the connection should send a close to
+ *			the remote end.  If false, don't inform the remote
+ *			end of the close.
+ * @return	Zero on success.
+ *		EINVAL if this handle was already closing or closed.  If the
+ * 			handle is closing, consider using ipmi_sol_force_close
+ *			to hurry it along.
+ *		A nonzero error code on other failure.
+ */
+int ipmi_sol_force_close_wsend(ipmi_sol_conn_t *conn, int rem_close);
 
 /**
  * Forces the closure of the SoL session.
@@ -544,8 +598,25 @@ int ipmi_sol_free(ipmi_sol_conn_t *conn);
  *		EINVAL	if a request is made to transmit zero bytes
  *		or a nonzero IPMI error code on failure.
  */
-int ipmi_sol_write(ipmi_sol_conn_t *conn, const void *buf, int count,
-	ipmi_sol_transmit_complete_cb cb, void *cb_data);
+int ipmi_sol_write(ipmi_sol_conn_t *conn,
+		   const void      *buf,
+		   int             count,
+		   ipmi_sol_transmit_complete_cb cb,
+		   void            *cb_data);
+
+
+/**
+ * Release a NACK
+ *
+ * This function will release a NACK returned by the receive routine.
+ * For every NACK a receive routine returns, this must be called to
+ * reactivate receiving packets from the BMC.
+ *
+ * @param [in] conn	The IPMI SoL connection over which to transmit.
+ * @return	Zero on success
+ *		EINVAL	No NACK is pending.
+ */
+int ipmi_sol_release_nack(ipmi_sol_conn_t *conn);
 
 
 /**
@@ -563,7 +634,8 @@ int ipmi_sol_write(ipmi_sol_conn_t *conn, const void *buf, int count,
  * 		data.
  */
 int ipmi_sol_send_break(ipmi_sol_conn_t *conn,
-	ipmi_sol_transmit_complete_cb cb, void *cb_data);
+			ipmi_sol_transmit_complete_cb cb,
+			void            *cb_data);
 
 
 /**
@@ -589,8 +661,10 @@ int ipmi_sol_send_break(ipmi_sol_conn_t *conn,
  *		this request unless it is sent at the same time as a block of
  * 		data.
  */
-int ipmi_sol_set_CTS_assertable(ipmi_sol_conn_t *conn, int asserted,
-	ipmi_sol_transmit_complete_cb cb, void *cb_data);
+int ipmi_sol_set_CTS_assertable(ipmi_sol_conn_t *conn,
+				int             asserted,
+				ipmi_sol_transmit_complete_cb cb,
+				void            *cb_data);
 
 
 /**
@@ -608,8 +682,10 @@ int ipmi_sol_set_CTS_assertable(ipmi_sol_conn_t *conn, int asserted,
  *		this request unless it is sent at the same time as a block of
  * 		data.
  */
-int ipmi_sol_set_DCD_DSR_asserted(ipmi_sol_conn_t *conn, int asserted,
-	ipmi_sol_transmit_complete_cb cb, void *cb_data);
+int ipmi_sol_set_DCD_DSR_asserted(ipmi_sol_conn_t *conn,
+				  int             asserted,
+				  ipmi_sol_transmit_complete_cb cb,
+				  void            *cb_data);
 
 
 /**
@@ -630,8 +706,10 @@ int ipmi_sol_set_DCD_DSR_asserted(ipmi_sol_conn_t *conn, int asserted,
  *		this request unless it is sent at the same time as a block of
  * 		data.
  */
-int ipmi_sol_set_RI_asserted(ipmi_sol_conn_t *conn, int asserted,
-	ipmi_sol_transmit_complete_cb cb, void *cb_data);
+int ipmi_sol_set_RI_asserted(ipmi_sol_conn_t *conn,
+			     int             asserted,
+			     ipmi_sol_transmit_complete_cb cb,
+			     void            *cb_data);
 
 
 /**
@@ -662,5 +740,13 @@ int ipmi_sol_set_RI_asserted(ipmi_sol_conn_t *conn, int asserted,
  *		this request unless it is sent at the same time as a block of
  * 		data.
  */
-int ipmi_sol_flush(ipmi_sol_conn_t *conn, int queue_selectors,
-	ipmi_sol_flush_complete_cb cb, void *cb_data);
+int ipmi_sol_flush(ipmi_sol_conn_t *conn,
+		   int             queue_selectors,
+		   ipmi_sol_flush_complete_cb cb,
+		   void            *cb_data);
+
+#ifdef __cplusplus
+}
+#endif
+
+#endif /* OPENIPMI_SOL_H */
