@@ -28,7 +28,24 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this program; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+ *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Modified BSD Licence
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *   1. Redistributions of source code must retain the above copyright
+ *      notice, this list of conditions and the following disclaimer.
+ *   2. Redistributions in binary form must reproduce the above
+ *      copyright notice, this list of conditions and the following
+ *      disclaimer in the documentation and/or other materials provided
+ *      with the distribution.
+ *   3. The name of the author may not be used to endorse or promote
+ *      products derived from this software without specific prior
+ *      written permission.
+ */
 
 #include "bmc.h"
 
@@ -54,6 +71,19 @@ get_lanserv_version(void)
     return PVERSION;
 }
 
+struct {
+    cmd_handler_f handler;
+    void *cb_data;
+} group_extension_handlers[256];
+
+void
+ipmi_emu_register_group_extension_handler(uint8_t group_extension,
+					  cmd_handler_f handler, void *cb_data)
+{
+    group_extension_handlers[group_extension].handler = handler;
+    group_extension_handlers[group_extension].cb_data = cb_data;
+}
+
 static void
 handle_group_extension_netfn(lmc_data_t    *mc,
 			     msg_t         *msg,
@@ -61,21 +91,17 @@ handle_group_extension_netfn(lmc_data_t    *mc,
 			     unsigned int  *rdata_len,
 			     void          *cb_data)
 {
+    uint8_t ge;
+
     if (check_msg_length(msg, 1, rdata, rdata_len))
 	return;
 
-    switch (msg->data[0]) {
-    case IPMI_PICMG_GRP_EXT:
-	if (mc->emu->atca_mode)
-	    handle_picmg_msg(mc, msg, rdata, rdata_len);
-	else
-	    handle_invalid_cmd(mc, rdata, rdata_len);
-	break;
-
-    default:
+    ge = msg->data[0];
+    if (group_extension_handlers[ge].handler)
+	group_extension_handlers[ge].handler(mc, msg, rdata, rdata_len,
+				     group_extension_handlers[ge].cb_data);
+    else
 	handle_invalid_cmd(mc, rdata, rdata_len);
-	break;
-    }
 }
 
 static struct iana_handler_elem {
@@ -244,7 +270,7 @@ ipmi_emu_register_cmd_handler(unsigned char netfn, unsigned char cmd,
 {
     unsigned int ni = netfn >> 1;
 
-    if (netfn >= 32)
+    if (ni >= 32)
 	return EINVAL;
 
     if (!netfn_handlers[ni].handlers) {
@@ -283,6 +309,11 @@ ipmi_emu_tick(emu_data_t *emu, unsigned int seconds)
     }
 }
 
+#define IPMI_SEND_MSG_NO_TRACK 0x0
+#define IPMI_SEND_MSG_TRACK_REQUEST 0x01
+#define IPMI_SEND_MSG_SEND_RAW 0x2
+#define IPMI_SEND_MSG_GET_TRACKING(v) ((v >> 6) & 0x3)
+
 void
 ipmi_emu_handle_msg(emu_data_t    *emu,
 		    lmc_data_t    *srcmc,
@@ -296,6 +327,7 @@ ipmi_emu_handle_msg(emu_data_t    *emu,
     unsigned char *data = NULL;
     unsigned char *rdata;
     unsigned int  *rdata_len;
+    channel_t *rchan = omsg->orig_channel;
 
     if (emu->sysinfo->debug & DEBUG_MSG)
 	emu->sysinfo->log(emu->sysinfo, DEBUG, omsg, "Receive message:");
@@ -312,6 +344,18 @@ ipmi_emu_handle_msg(emu_data_t    *emu,
 	    return;
 	}
 
+	switch (IPMI_SEND_MSG_GET_TRACKING(omsg->data[0])) {
+	case IPMI_SEND_MSG_NO_TRACK:
+	    rchan = srcmc->channels[15];
+	    break;
+	case IPMI_SEND_MSG_TRACK_REQUEST:
+	    break;
+	default:
+	    ordata[0] = IPMI_INVALID_DATA_FIELD_CC;
+	    *ordata_len = 1;
+	    return;
+	}
+
 	data = omsg->data + 1;
 	data_len = omsg->len - 1;
 	if (data[0] == 0) {
@@ -396,10 +440,9 @@ ipmi_emu_handle_msg(emu_data_t    *emu,
 
     if (omsg->netfn == IPMI_APP_NETFN && omsg->cmd == IPMI_SEND_MSG_CMD) {
 	/* An encapsulated command, put the response into the receive q. */
-	channel_t *bchan = srcmc->channels[15];
 
-	if (bchan->recv_in_q) {
-	    if (bchan->recv_in_q(srcmc->channels[15], rmsg))
+	if (rchan->recv_in_q) {
+	    if (rchan->recv_in_q(srcmc->channels[15], rmsg))
 		return;
 	}
 
@@ -410,37 +453,52 @@ ipmi_emu_handle_msg(emu_data_t    *emu,
 	    debug_log_raw_msg(emu->sysinfo, rdata, *rdata_len,
 			      "Response message:");
 
+	if (!rchan->has_recv_q) {
+	    free(rmsg);
+	    return;
+	}
+
 	rmsg->len += 6;
 	rmsg->data[rmsg->len] = -ipmb_checksum(rmsg->data, rmsg->len, 0);
 	rmsg->len += 1;
-	if (srcmc->recv_q_tail) {
-	    rmsg->next = srcmc->recv_q_tail;
-	    srcmc->recv_q_tail = rmsg;
+	if (rchan->recv_q_tail) {
+	    rmsg->next = rchan->recv_q_tail;
+	    rchan->recv_q_tail = rmsg;
 	} else {
-	    channel_t *bchan = srcmc->channels[15];
-
 	    rmsg->next = NULL;
-	    srcmc->recv_q_head = rmsg;
-	    srcmc->recv_q_tail = rmsg;
-	    if (bchan->set_atn)
-		bchan->set_atn(bchan, 1, IPMI_MC_MSG_INTS_ON(mc));
+	    rchan->recv_q_head = rmsg;
+	    rchan->recv_q_tail = rmsg;
+	    if (rchan->channel_num == 15) {
+		srcmc->msg_flags |= IPMI_MC_MSG_FLAG_RCV_MSG_QUEUE;
+		if (rchan->set_atn)
+		    rchan->set_atn(rchan, 1, IPMI_MC_MSG_INTS_ON(mc));
+	    }
 	}
     } else if (emu->sysinfo->debug & DEBUG_MSG)
 	debug_log_raw_msg(emu->sysinfo, ordata, *ordata_len,
 			  "Response message:");
 }
 
+void
+ipmi_resend_atn(channel_t *chan)
+{
+    lmc_data_t *mc = chan->mc;
+
+    if (chan->set_atn)
+	chan->set_atn(chan, !!mc->msg_flags, IPMI_MC_MSG_INTS_ON(mc));
+}
+
 msg_t *
-ipmi_mc_get_next_recv_q(lmc_data_t *mc)
+ipmi_mc_get_next_recv_q(channel_t *chan)
 {
     msg_t *rv;
 
-    if (!mc->recv_q_head)
+    if (!chan->recv_q_head)
 	return NULL;
-    rv = mc->recv_q_head;
-    mc->recv_q_head = rv->next;
-    if (!mc->recv_q_head) {
-	mc->recv_q_tail = NULL;
+    rv = chan->recv_q_head;
+    chan->recv_q_head = rv->next;
+    if (!chan->recv_q_head) {
+	chan->recv_q_tail = NULL;
     }
     return rv;
 }
@@ -564,6 +622,9 @@ ipmi_mc_enable(lmc_data_t *mc)
 	    err = sys->lan_channel_init(sys->info, chan);
 	else if (chan->medium_type == IPMI_CHANNEL_MEDIUM_RS232)
 	    err = sys->ser_channel_init(sys->info, chan);
+	else if ((chan->medium_type == IPMI_CHANNEL_MEDIUM_IPMB) &&
+		((chan->channel_num != 0) || (chan->prim_ipmb_in_cfg_file)))
+	    err = sys->ipmb_channel_init(sys->info, chan);
 	else 
 	    chan_init(chan);
 	if (err) {
@@ -598,10 +659,8 @@ init_mc(emu_data_t *emu, lmc_data_t *mc, unsigned int persist_sdr)
 
     err = mc->sysinfo->alloc_timer(mc->sysinfo, watchdog_timeout,
 				   mc, &mc->watchdog_timer);
-    if (err) {
-	free(mc);
+    if (err)
 	return err;
-    }
 
     if (persist_sdr && mc->has_device_sdrs) {
 	read_mc_sdrs(mc, &mc->device_sdrs[0], "device0");
@@ -755,6 +814,7 @@ ipmi_mc_alloc_unconfigured(sys_data_t *sys, unsigned char ipmb,
     mc->ipmb_channel.protocol_type = IPMI_CHANNEL_PROTOCOL_IPMB;
     mc->ipmb_channel.session_support = IPMI_CHANNEL_SESSION_LESS;
     mc->ipmb_channel.active_sessions = 0;
+    mc->ipmb_channel.prim_ipmb_in_cfg_file = 0;
     mc->channels[0] = &mc->ipmb_channel;
     mc->channels[0]->log = sys->clog;
 
@@ -797,9 +857,7 @@ handle_tick(void *info, unsigned int seconds)
 	    /* Waiting for the first kill */
 	    mc->startcmd.wait_poweroff--;
 	    if (mc->startcmd.wait_poweroff == 0) {
-		if (HW_OP_CAN_POWER(mc->channels[15]))
-		    mc->channels[15]->hw_op(mc->channels[15], HW_OP_FORCEOFF);
-		else if (mc->startcmd.vmpid)
+		if (mc->startcmd.vmpid)
 		    ipmi_do_kill(&mc->startcmd, 0);
 		mc->startcmd.wait_poweroff = -mc->startcmd.kill_wait_time;
 	    }
@@ -899,6 +957,8 @@ ipmi_emu_add_mc(emu_data_t    *emu,
     }
 
     if (ipmb == emu->sysinfo->bmc_ipmb) {
+	int rv;
+
 	if (!mc->channels[15]) {
 	    /* No one specified a system channel, make one up */
 	    mc->sys_channel.medium_type = IPMI_CHANNEL_MEDIUM_SYS_INTF;
@@ -909,8 +969,13 @@ ipmi_emu_add_mc(emu_data_t    *emu,
 	    mc->channels[15] = &mc->sys_channel;
 	}
 
+	mc->channels[15]->has_recv_q = 1;
 	mc->sysinfo = emu->sysinfo;
-	init_mc(emu, mc, flags & IPMI_MC_PERSIST_SDR);
+	rv = init_mc(emu, mc, flags & IPMI_MC_PERSIST_SDR);
+	if (rv) {
+	    free(mc);
+	    return rv;
+	}
     }
 
     if (mc->startcmd.startcmd) {
