@@ -28,7 +28,24 @@
  *
  *  You should have received a copy of the GNU Lesser General Public
  *  License along with this program; if not, write to the Free
- *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+ *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ * Modified BSD Licence
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ *   1. Redistributions of source code must retain the above copyright
+ *      notice, this list of conditions and the following disclaimer.
+ *   2. Redistributions in binary form must reproduce the above
+ *      copyright notice, this list of conditions and the following
+ *      disclaimer in the documentation and/or other materials provided
+ *      with the distribution.
+ *   3. The name of the author may not be used to endorse or promote
+ *      products derived from this software without specific prior
+ *      written permission.
+ */
 
 #include "bmc.h"
 
@@ -227,11 +244,15 @@ watchdog_timeout(void *cb_data)
     case IPMI_MC_WATCHDOG_ACTION_POWER_DOWN:
 	set_sensor_bit(mc, sens, 2, 1, 0xc2, mc->watchdog_use & 0xf, 0xff, 1);
 	bchan->hw_op(bchan, HW_OP_POWEROFF);
+	if (bchan->stop_cmd)
+	    bchan->stop_cmd(bchan, 0);
 	break;
 
     case IPMI_MC_WATCHDOG_ACTION_POWER_CYCLE:
 	set_sensor_bit(mc, sens, 3, 1, 0xc3, mc->watchdog_use & 0xf, 0xff, 1);
 	bchan->hw_op(bchan, HW_OP_POWEROFF);
+	if (bchan->stop_cmd)
+	    bchan->stop_cmd(bchan, 0);
 	start_poweron_timer(mc);
 	break;
     }
@@ -524,7 +545,7 @@ handle_set_global_enables(lmc_data_t    *mc,
 	bchan->hw_op(bchan, HW_OP_IRQ_DISABLE);
 
     if ((!old_evint && IPMI_MC_EVBUF_FULL_INT_ENABLED(mc) && mc->ev_in_q) ||
-	(old_int && !IPMI_MC_MSG_INTS_ON(mc) && mc->recv_q_tail))
+	(old_int && !IPMI_MC_MSG_INTS_ON(mc) && mc->channels[15]->recv_q_tail))
 	bchan->set_atn(bchan, 1, IPMI_MC_EVBUF_FULL_INT_ENABLED(mc));
 }
 
@@ -888,7 +909,8 @@ handle_read_event_msg_buffer(lmc_data_t    *mc,
     mc->ev_in_q = 0;
     mc->msg_flags &= ~IPMI_MC_MSG_FLAG_EVT_BUF_FULL;
     if (chan->set_atn)
-	chan->set_atn(chan, 0, IPMI_MC_EVBUF_FULL_INT_ENABLED(mc));
+	chan->set_atn(chan, !!mc->msg_flags,
+		      IPMI_MC_EVBUF_FULL_INT_ENABLED(mc));
 }
 
 static void
@@ -910,10 +932,29 @@ handle_clear_msg_flags(lmc_data_t    *mc,
 		       unsigned int  *rdata_len,
 		       void          *cb_data)
 {
+    channel_t *chan = mc->channels[15];
+
     if (check_msg_length(msg, 1, rdata, rdata_len))
 	return;
 
     mc->msg_flags &= ~msg->data[0];
+
+    if (msg->data[0] & IPMI_MC_MSG_FLAG_EVT_BUF_FULL)
+	mc->ev_in_q = 0;
+
+    if (msg->data[0] & IPMI_MC_MSG_FLAG_RCV_MSG_QUEUE) {
+	while (chan->recv_q_head) {
+	    msg_t *qmsg = chan->recv_q_head;
+
+	    chan->recv_q_head = qmsg->next;
+	    free(qmsg);
+	}
+	chan->recv_q_tail = NULL;
+    }
+
+    if (chan->set_atn)
+	chan->set_atn(chan, !!mc->msg_flags, IPMI_MC_MSG_INTS_ON(mc));
+
     rdata[0] = 0;
     *rdata_len = 1;
 }
@@ -926,6 +967,7 @@ handle_get_msg(lmc_data_t    *mc,
 	       void          *cb_data)
 {
     msg_t *qmsg;
+    channel_t *chan = mc->channels[15];
 
     if (!mc->sysinfo) {
 	rdata[0] = IPMI_INVALID_CMD_CC;
@@ -933,7 +975,7 @@ handle_get_msg(lmc_data_t    *mc,
 	return;
     }
 
-    qmsg = mc->recv_q_head;
+    qmsg = chan->recv_q_head;
     if (!qmsg) {
 	rdata[0] = 0x80;
 	*rdata_len = 1;
@@ -946,12 +988,12 @@ handle_get_msg(lmc_data_t    *mc,
 	return;
     }
 
-    mc->recv_q_head = qmsg->next;
+    chan->recv_q_head = qmsg->next;
     if (!qmsg->next) {
-	channel_t *bchan = mc->channels[15];
-	mc->recv_q_tail = NULL;
-	if (bchan->set_atn)
-	    bchan->set_atn(bchan, 0, IPMI_MC_MSG_INTS_ON(mc));
+	chan->recv_q_tail = NULL;
+	mc->msg_flags &= ~IPMI_MC_MSG_FLAG_RCV_MSG_QUEUE;
+	if (chan->set_atn)
+	    chan->set_atn(chan, !!mc->msg_flags, IPMI_MC_MSG_INTS_ON(mc));
     }
 
     rdata[0] = 0;
