@@ -441,7 +441,7 @@ ilanserv_log(sys_data_t *sys, int logtype, msg_t *msg, const char *format,
 #define mformat " channel=%d netfn=0x%x cmd=0x%x rs_addr=0x%x rs_lun=0x%x" \
 	    " rq_addr=0x%x\n rq_lun=0x%x rq_seq=0x%x\n"
 
-	len += snprintf(&dummy, 1, mformat, msg->channel, msg->netfn,
+	len += snprintf(&dummy, 0, mformat, msg->channel, msg->netfn,
 			msg->cmd, msg->rs_addr, msg->rs_lun, msg->rq_addr,
 			msg->rq_lun, msg->rq_seq);
 	len += 3 * msg->len + 3;
@@ -611,8 +611,13 @@ sys_start_cmd(sys_data_t *sys)
 {
 }
 
+void
+ipmi_resend_atn(channel_t *chan)
+{
+}
+
 msg_t *
-ipmi_mc_get_next_recv_q(lmc_data_t *mc)
+ipmi_mc_get_next_recv_q(channel_t *chan)
 {
     return NULL;
 }
@@ -858,7 +863,7 @@ main(int argc, const char *argv[])
 		lan->lan_addr_set = 1;
 	    }
 
-	    lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr,
+	    lan_fd = open_lan_fd(&lan->lan_addr.addr.s_ipsock.s_addr0,
 				 lan->lan_addr.addr_len);
 	    if (lan_fd == -1) {
 		fprintf(stderr, "Unable to open LAN address %d\n", i+1);
