@@ -248,7 +248,7 @@ raw_send(lanserv_data_t *lan,
 			  "Raw LAN send to:");
 	for (i = 0; i < vecs; i++)
 	    len += vec[i].iov_len;
-	slen = snprintf(&dummy, 1, format);
+	slen = snprintf(&dummy, 0, format);
 	slen += len * 3 + 3;
 	str = malloc(slen);
 	if (!str)
@@ -310,6 +310,7 @@ return_rmcpp_rsp(lanserv_data_t *lan, session_t *session, msg_t *msg,
 	pos[5] = msg->cmd;
 	pos[len] = -ipmb_checksum(pos+3, len-3, 0);
 	len++;
+	dlen++;
     }
 
     if (session && !session->in_startup) {
@@ -2637,7 +2638,7 @@ handle_rakp1_payload(lanserv_data_t *lan, msg_t *msg)
     if (!user) {
 	lan->sysinfo->log(lan->sysinfo, NEW_SESSION_FAILED, msg,
 		 "RAKP msg: invalid user: %s", username);
-	err = IPMI_RMCPP_ILLEGAL_PARAMETER;
+	err = IPMI_RMCPP_UNAUTHORIZED_NAME;
 	goto out_err;
     }
 
