@@ -34,6 +34,7 @@
 #include <stdint.h>
 #include <string.h>
 #include <ctype.h>
+#include <unistd.h>
 #include <stdlib.h>
 #include <malloc.h>
 #include <math.h>
@@ -43,18 +44,33 @@
 #include <OpenIPMI/ipmi_string.h>
 
 #include "persist.c"
+#include "string.c"
 
 #define MAX_SDR_LINE 256
 
+static FILE *outfile;
+static char *outfname = NULL;
+
+static void
+out_err(int err)
+{
+    if (outfname) {
+	unlink(outfname);
+    }
+
+    exit(err);
+}
+
 struct sdr_field_name {
     char *name;
-    unsigned int val;
+    int val;
 };
 
 struct sdr_field {
     char *name;
+    /* SBITS means signed bits, the highest bit is a sign bit, 2s complement */
     enum { SDR_BITS, SDR_SBITS, SDR_MULTIBITS, SDR_MULTISBITS, SDR_MULTIBITS2,
-	   SDR_STRING, SDR_BOOLBIT, SDR_THRESH } type;
+	   SDR_STRING, SDR_BOOLBIT, SDR_THRESH, SDR_THRESHREL } type;
     /*
      * IMPORTANT: pos is offset + 1, the values given in the IPMI spec.
      * It is not zero-based.
@@ -120,7 +136,11 @@ static struct sdr_field_name entity_id_fields[] = {
     { "pci_express_bus", 49 },
     { "scsi_bus", 50 },
     { "sata_/_sas_bus", 51 },
-    { "processor_/_front-side_bus", 52 }
+    { "processor_/_front-side_bus", 52 },
+    { "picmg_front", 0xa0 },
+    { "picmg_rtm", 0xc0 },
+    { "picmg_amc", 0xc1 },
+    { NULL }
 };
 
 static struct sdr_field_name sensor_type_fields[] = {
@@ -167,7 +187,11 @@ static struct sdr_field_name sensor_type_fields[] = {
     { "Battery", 41 },
     { "Session_Audit", 42 },
     { "Version_Change", 43 },
-    { "FRU_State", 44 }
+    { "FRU_State", 44 },
+    { "picmg_FruHotSwap", 0xF0 },
+    { "picmg_IPMB0", 0xF1 },
+    { "picmg_ModuleHotSwap", 0xF2 },
+    { NULL }
 };
 
 static struct sdr_field_name sensor_access_fields[] = {
@@ -289,7 +313,8 @@ static struct sdr_field_name base_unit_fields[] = {
     { "characters", 87 },
     { "error", 88 },
     { "correctable_error", 89 },
-    { "uncorrectable_error", 90 }
+    { "uncorrectable_error", 90 },
+    { NULL }
 };
 
 static struct sdr_field_name linearization_fields[] = {
@@ -414,7 +439,7 @@ static struct sdr_field type1[] =
     { "unr_thrsh_settable",	SDR_BOOLBIT,	20, 5, 1 },
     { "uc_thrsh_settable",	SDR_BOOLBIT,	20, 4, 1 },
     { "unc_thrsh_settable",	SDR_BOOLBIT,	20, 3, 1 },
-    { "lnr_thresh_settable",	SDR_BOOLBIT,	20, 2, 1 },
+    { "lnr_thrsh_settable",	SDR_BOOLBIT,	20, 2, 1 },
     { "lc_thrsh_settable",	SDR_BOOLBIT,	20, 1, 1 },
     { "lnc_thrsh_settable",	SDR_BOOLBIT,	20, 0, 1 },
     { "unr_thrsh_readable",	SDR_BOOLBIT,	19, 5, 1 },
@@ -455,7 +480,9 @@ static struct sdr_field type1[] =
     { "nominal_reading",	SDR_BITS,	32, 0, 8 },
     { "nominal_freading",	SDR_THRESH,	32, 0, 8 },
     { "normal_maximum",		SDR_BITS,	33, 0, 8 },
+    { "normal_fmaximum",	SDR_THRESH,	33, 0, 8 },
     { "normal_minimum",		SDR_BITS,	34, 0, 8 },
+    { "normal_fminimum",	SDR_THRESH,	34, 0, 8 },
     { "sensor_maximum",		SDR_BITS,	35, 0, 8 },
     { "sensor_minimum",		SDR_BITS,	36, 0, 8 },
     { "unr_thresh",		SDR_BITS,	37, 0, 8 },
@@ -471,7 +498,9 @@ static struct sdr_field type1[] =
     { "lc_fthresh",		SDR_THRESH,	41, 0, 8 },
     { "lnc_fthresh",		SDR_THRESH,	42, 0, 8 },
     { "positive_hysteresis",	SDR_BITS,	43, 0, 8 },
+    { "positive_fhysteresis",	SDR_THRESHREL,	43, 0, 8 },
     { "negative_hysteresis",	SDR_BITS,	44, 0, 8 },
+    { "negative_fhysteresis",	SDR_THRESHREL,	44, 0, 8 },
     { "oem",			SDR_BITS,	47, 0, 8 },
     { "id_string",		SDR_STRING,	48, 0, 8, .required = 1 },
 };
@@ -590,7 +619,7 @@ static struct sdr_field type2[] =
     { "unr_thrsh_settable",	SDR_BOOLBIT,	20, 5, 1 },
     { "uc_thrsh_settable",	SDR_BOOLBIT,	20, 4, 1 },
     { "unc_thrsh_settable",	SDR_BOOLBIT,	20, 3, 1 },
-    { "lnr_thresh_settable",	SDR_BOOLBIT,	20, 2, 1 },
+    { "lnr_thrsh_settable",	SDR_BOOLBIT,	20, 2, 1 },
     { "lc_thrsh_settable",	SDR_BOOLBIT,	20, 1, 1 },
     { "lnc_thrsh_settable",	SDR_BOOLBIT,	20, 0, 1 },
     { "unr_thrsh_readable",	SDR_BOOLBIT,	19, 5, 1 },
@@ -608,7 +637,7 @@ static struct sdr_field type2[] =
       .strvals = modifier_unit_fields },
     { "percentage",		SDR_BOOLBIT,	21, 0, 1 },
     { "base_unit",		SDR_BITS,	22, 0, 8 },
-    { "modifier_unit",		SDR_BITS,	23, 0, 8 },
+    { "modifier_unit_code",	SDR_BITS,	23, 0, 8 },
     { "sensor_direction",	SDR_BITS,	24, 6, 2,
       .strvals = sensor_direction_fields },
     { "id_string_modifier",	SDR_BITS,	24, 4, 2,
@@ -655,7 +684,7 @@ static struct sdr_field type8[] =
     { "container_entity_inst",	SDR_BITS,	 7, 0, 8, .required = 1 },
     { "entities_are_range",	SDR_BOOLBIT,	 8, 7, 1 },
     { "linked_ears",		SDR_BOOLBIT,	 8, 6, 1 },
-    { "presense_sensor_always_there",SDR_BOOLBIT, 8, 5, 1 },
+    { "presence_sensor_always_there",SDR_BOOLBIT, 8, 5, 1 },
 
     { "entity_1_id",		SDR_BITS,	 9, 0, 8, .required = 1,
       .strvals = entity_id_fields },
@@ -681,7 +710,7 @@ static struct sdr_field type9[] =
     { "container_entity_dev_chan", SDR_BITS,	 9, 0, 8, .required = 1 },
     { "entities_are_range",	SDR_BOOLBIT,	 10, 7, 1 },
     { "linked_ears",		SDR_BOOLBIT,	 10, 6, 1 },
-    { "presense_sensor_always_there",SDR_BOOLBIT, 10, 5, 1 },
+    { "presence_sensor_always_there",SDR_BOOLBIT, 10, 5, 1 },
     { "entity_1_dev_addr",	SDR_BITS,	 11, 0, 8, .required = 1 },
     { "entity_1_dev_chan",	SDR_BITS,	 12, 0, 8, .required = 1 },
     { "entity_1_id",		SDR_BITS,	 13, 0, 8, .required = 1,
@@ -903,7 +932,7 @@ isquote(char c)
 }
 
 int
-get_delim_str(char **rtokptr, const char **rval, char **err)
+get_delim_str(char **rtokptr, char **rval, char **err)
 {
     char *tokptr = *rtokptr;
     char endc;
@@ -930,8 +959,12 @@ get_delim_str(char **rtokptr, const char **rval, char **err)
 	    oldc = *tokptr;
 	    *tokptr = '\0';
 	    val = find_var(val);
-	    if (!val)
+	    if (!val) {
+		if (rv)
+		    free(rv);
+		*err = "variable not found";
 		return -1;
+	    }
 	    *tokptr = oldc;
 	} else if (isquote(*tokptr)) {
 	    endc = *tokptr;
@@ -939,6 +972,8 @@ get_delim_str(char **rtokptr, const char **rval, char **err)
 	    val = tokptr;
 	    while (*tokptr != endc) {
 		if (*tokptr == '\0') {
+		    if (rv)
+			free(rv);
 		    *err = "End of line in string";
 		    return -1;
 		}
@@ -947,6 +982,8 @@ get_delim_str(char **rtokptr, const char **rval, char **err)
 	    *tokptr = '\0';
 	    tokptr++;
 	} else {
+	    if (rv)
+		free(rv);
 	    *err = "string value must start with '\"' or '''";
 	    return -1;
 	}
@@ -1178,6 +1215,69 @@ get_sdr_bits(unsigned char *sdr, unsigned int pos, unsigned int bitoff,
     return (sdr[pos - 1] >> bitoff) & ((1 << len) - 1);
 }
 
+int
+get_sdr_type(int type, struct sdr_field **t, unsigned int *tlen,
+	     unsigned int *sdr_len, char **errstr)
+{
+    switch (type) {
+    case 1:
+	*t = type1;
+	*tlen = TYPE1_LEN;
+	*sdr_len = 48;
+	break;
+
+    case 2:
+	*t = type2;
+	*tlen = TYPE2_LEN;
+	*sdr_len = 32;
+	break;
+
+    case 3:
+	*t = type3;
+	*tlen = TYPE3_LEN;
+	*sdr_len = 17;
+	break;
+
+    case 8:
+	*t = type8;
+	*tlen = TYPE8_LEN;
+	*sdr_len = 16;
+	break;
+
+    case 9:
+	*t = type9;
+	*tlen = TYPE9_LEN;
+	*sdr_len = 32;
+	break;
+
+    case 16:
+	*t = type16;
+	*tlen = TYPE16_LEN;
+	*sdr_len = 16;
+	break;
+
+    case 17:
+	*t = type17;
+	*tlen = TYPE17_LEN;
+	*sdr_len = 16;
+	break;
+
+    case 18:
+	*t = type18;
+	*tlen = TYPE18_LEN;
+	*sdr_len = 16;
+	break;
+
+    default:
+	if (errstr)
+	    *errstr = "Unknown SDR type, supported types are 1, 2, 3, 8, 9,"
+		" 16 (0x10) and 17 (0x11)";
+	return -1;
+    }
+
+    return 0;
+}
+
 int
 ipmi_compile_sdr(FILE *f, unsigned int type,
 		 unsigned char **retbuf, unsigned int *retlen,
@@ -1194,60 +1294,8 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 
     *errstr2 = NULL;
 
-    switch (type) {
-    case 1:
-	t = type1;
-	tlen = TYPE1_LEN;
-	sdr_len = 48;
-	break;
-
-    case 2:
-	t = type2;
-	tlen = TYPE2_LEN;
-	sdr_len = 32;
-	break;
-
-    case 3:
-	t = type3;
-	tlen = TYPE3_LEN;
-	sdr_len = 17;
-	break;
-
-    case 8:
-	t = type8;
-	tlen = TYPE8_LEN;
-	sdr_len = 16;
-	break;
-
-    case 9:
-	t = type9;
-	tlen = TYPE9_LEN;
-	sdr_len = 32;
-	break;
-
-    case 16:
-	t = type16;
-	tlen = TYPE16_LEN;
-	sdr_len = 16;
-	break;
-
-    case 17:
-	t = type17;
-	tlen = TYPE17_LEN;
-	sdr_len = 16;
-	break;
-
-    case 18:
-	t = type18;
-	tlen = TYPE18_LEN;
-	sdr_len = 16;
-	break;
-
-    default:
-	*errstr = "Unknown SDR type, supported types are 1, 2, 3, 8, 9,"
-	    " 16 (0x10) and 17 (0x11)";
+    if (get_sdr_type(type, &t, &tlen, &sdr_len, errstr))
 	return -1;
-    }
 
     requireds = malloc(tlen * sizeof(char));
     if (!requireds) {
@@ -1323,7 +1371,7 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 		err = get_uint_str(&t[i], &tokptr, &uval, errstr);
 		if (err)
 		    goto out_err;
-		umax = 1 << t[i].bitsize;
+		umax = (1 << t[i].bitsize) - 1;
 		if (uval > umax) {
 		    err = -1;
 		    *errstr = "Value too large for bit size";
@@ -1342,7 +1390,7 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 		err = get_int(&tokptr, &sval, errstr);
 		if (err)
 		    goto out_err;
-		smax = 1 << (t[i].bitsize - 1);
+		smax = (1 << (t[i].bitsize - 1)) - 1;
 		smin = -smax - 1;
 		if (sval > smax || sval < smin) {
 		    err = -1;
@@ -1419,7 +1467,7 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 	    case SDR_STRING:
 	    {
 		unsigned char str[IPMI_MAX_STR_LEN];
-		const char *sval;
+		char *sval;
 		unsigned int out_len = sizeof(str);
 
 		err = get_delim_str(&tokptr, &sval, errstr);
@@ -1427,6 +1475,7 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 		    goto out_err;
 		ipmi_set_device_string(sval, IPMI_ASCII_STR, strlen(sval),
 				       str, 0, &out_len);
+		free(sval);
 		if (out_len > 1) {
 		    unsigned char *newsdr = realloc(sdr, sdr_len + out_len - 1);
 		    if (!newsdr) {
@@ -1448,6 +1497,7 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 		break;
 
 	    case SDR_THRESH:
+	    case SDR_THRESHREL:
 	    {
 		double fval, fx;
 		int m, b, r_exp, b_exp;
@@ -1459,25 +1509,47 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
 		m = get_sdr_bits(sdr, 25, 0, 8);
 		m |= get_sdr_bits(sdr, 26, 6, 2) << 8;
 		if (m & (1 << 9))
-		    m |= (~0 << 10);
+		    m |= (~0U << 10);
 		b = get_sdr_bits(sdr, 27, 0, 8);
 		b |= get_sdr_bits(sdr, 28, 6, 2) << 8;
 		if (b & (1 << 9))
-		    b |= (~0 << 10);
+		    b |= (~0U << 10);
 		r_exp = get_sdr_bits(sdr, 30, 4, 4);
 		if (r_exp & (1 << 3))
-		    r_exp |= (~0 << 4);
+		    r_exp |= (~0U << 4);
 		b_exp = get_sdr_bits(sdr, 30, 0, 4);
 		if (b_exp & (1 << 3))
-		    b_exp |= (~0 << 4);
+		    b_exp |= (~0U << 4);
 
-		fx = (((fval / pow(10, r_exp)) - ((double) b) * pow(10, b_exp))
-		      / ((double) m));
+		if (t[i].type == SDR_THRESHREL)
+		    /*
+		     * A threshold value, like hysteresis.  It's not
+		     * an absolute value, it's a relative value, so we
+		     * leave "b" out of the equation.  Note that this
+		     * only works for linear equations.
+		     */
+		    fx = (((fval / pow(10, r_exp))) / ((double) m));
+		else
+		    fx = (((fval / pow(10, r_exp))
+			   - ((double) b) * pow(10, b_exp))
+			  / ((double) m));
 
-		if (t[i].name[0] == 'u')
-		    fx = ceil(fx);
-		else if (t[i].name[0] != 'l')
-		    fx += .5; /* round */
+		/*
+		 * We always round here.  This means that a threshold
+		 * may be set that is on the "wrong side" of the
+		 * threshold, and the trigger may too sensitive.  Or
+		 * perhaps insensitive.  Of course, that can happen
+		 * without rounding, too.  There appears to be no easy
+		 * way to second-guess the user of this program and,
+		 * given a floating point value, figure out the proper
+		 * integer value they want. Instead, they are expected
+		 * to give a floating point value that will evaluate
+		 * very closely to the integer they want.  Rounding
+		 * should handle the issue of getting it right on the
+		 * integer value the user wants and avoid issues with
+		 * floating point imprecision.
+		 */
+		fx = round(fx);
 
 		if (fx < 0.0 || fx > 255.0) {
 		    err = -1;
@@ -1518,17 +1590,9 @@ ipmi_compile_sdr(FILE *f, unsigned int type,
     return err;
 }
 
-static char *progname;
-
-static void help(void)
-{
-    fprintf(stderr, "%s [-r] <input file>\n", progname);
-    exit(1);
-}
-
 static void
-parse_file(const char *filename, FILE *f, persist_t *p, int outraw,
-	   unsigned int *sdrnum)
+compile_file(const char *filename, FILE *f, persist_t *p, int outraw,
+	     unsigned int *sdrnum)
 {
     char buf[MAX_SDR_LINE];
     char *s;
@@ -1554,7 +1618,7 @@ parse_file(const char *filename, FILE *f, persist_t *p, int outraw,
 		fprintf(stderr,
 			"%s:%3d: Invalid input, expecting \"sdr type <n>\"\n",
 			filename, line);
-		exit(1);
+		out_err(1);
 	    }
 
 	    err = get_uint(&tokptr, &sdrtype, &errstr, NULL);
@@ -1563,7 +1627,7 @@ parse_file(const char *filename, FILE *f, persist_t *p, int outraw,
 			"%s:%3d: Invalid input, expecting \"sdr type <n>\":"
 			" %s\n",
 			filename, line, errstr);
-		exit(1);
+		out_err(1);
 	    }
 
 	    err = ipmi_compile_sdr(f, sdrtype, &sdr, &sdrlen, &errstr, &errstr2,
@@ -1574,32 +1638,32 @@ parse_file(const char *filename, FILE *f, persist_t *p, int outraw,
 			    errstr, errstr2);
 		else
 		    fprintf(stderr, "%s:%3d: %s\n", filename, line, errstr);
-		exit(1);
+		out_err(1);
 	    }
 
 	    sdr[0] = *sdrnum & 0xff;
 	    sdr[1] = (*sdrnum >> 8) & 0xff;
 
 	    if (outraw) {
-		fwrite(sdr, sdrlen, 1, stdout);
+		fwrite(sdr, sdrlen, 1, outfile);
 	    } else {
 		err = add_persist_data(p, sdr, sdrlen, "%d", *sdrnum);
 		if (err) {
 		    fprintf(stderr, "Out of memory\n");
-		    exit(1);
+		    out_err(1);
 		}
 	    }
 	    (*sdrnum)++;
 	    free(sdr);
 	} else if (strcmp(tok, "define") == 0) {
 	    const char *name;
-	    const char *value;
+	    char *value;
 	    name = mystrtok(NULL, " \n\t", &tokptr);
 	    if (!name) {
 		fprintf(stderr,
 			"%s:%3d: Invalid input, expecting variable name\n",
 			filename, line);
-		exit(1);
+		out_err(1);
 	    }
 
 	    err = get_delim_str(&tokptr, &value, &errstr);
@@ -1607,14 +1671,16 @@ parse_file(const char *filename, FILE *f, persist_t *p, int outraw,
 		fprintf(stderr,
 			"%s:%3d: Invalid value, expecting quote delimited"
 			" string: %s\n", filename, line, errstr);
-		exit(1);
+		out_err(1);
 	    }
 	    
 	    err = add_variable(name, value);
-	    if (err)
-		exit(1);
+	    if (err) {
+		free(value);
+		out_err(1);
+	    }
 	} else if (strcmp(tok, "include") == 0) {
-	    const char *nfilename;
+	    char *nfilename;
 	    FILE *f2;
 
 	    err = get_delim_str(&tokptr, &nfilename, &errstr);
@@ -1622,29 +1688,273 @@ parse_file(const char *filename, FILE *f, persist_t *p, int outraw,
 		fprintf(stderr,
 			"%s:%3d: Invalid filename, expecting quote delimited"
 			" string: %s\n", filename, line, errstr);
-		exit(1);
+		out_err(1);
 	    }
 
 	    f2 = fopen(nfilename, "r");
 	    if (!f2) {
 		fprintf(stderr, "%s:%3d: Unable to open included file %s\n",
 			filename, line, nfilename);
-		exit(1);
+		out_err(1);
 	    }
 
-	    parse_file(nfilename, f2, p, outraw, sdrnum);
+	    compile_file(nfilename, f2, p, outraw, sdrnum);
 	    
 	    fclose(f2);
 	} else {
 	    fprintf(stderr, "%s:%3d: Invalid input,"
 		    " expecting \"sdr type <n>\"\n",
 		    filename, line);
-	    exit(1);
+	    out_err(1);
 	}
     }
 
 }
 
+static int
+extract_bits(uint8_t *sdr, struct sdr_field *t)
+{
+    unsigned int v;
+
+    /*
+     * Note that the pos is the IPMI 1's based offset, not a zero
+     * based offset.  So we subtract 1 to get the array position.
+     */
+    v = sdr[t->pos - 1];
+    v >>= t->bitoff;
+    v &= (1 << t->bitsize) - 1;
+    return v;
+}
+
+static int
+sign_extend(int v, int bitsize)
+{
+    if (v & (1 << (bitsize - 1))) {
+	/* Negative */
+	v &= ~(1 << (bitsize - 1));
+	v = -v - 1;
+    }
+
+    return v;
+}
+
+static void
+dump_val(struct sdr_field *t, int v)
+{
+    struct sdr_field_name *n = t->strvals;
+
+    if (n) {
+	while (n->name) {
+	    if (v == n->val) {
+		fprintf(outfile, "%s\n", n->name);
+		return;
+	    }
+	    n++;
+	}
+    }
+    fprintf(outfile, "%d\n", v);
+}
+
+static void
+dump_int(uint8_t *sdr, struct sdr_field *t, int is_signed)
+{
+    int v = extract_bits(sdr, t);
+
+    if (is_signed)
+	v = sign_extend(v, t->bitsize);
+
+    dump_val(t, v);
+}
+
+static void
+dump_multibits(uint8_t *sdr, struct sdr_field *t, int is_signed)
+{
+    int v = extract_bits(sdr, t);
+    int shift = t->bitsize;
+
+    t++;
+    while (t->type == SDR_MULTIBITS2) {
+	v |= extract_bits(sdr, t) << shift;
+	shift += t->bitsize;
+	t++;
+    }
+    if (is_signed)
+	v = sign_extend(v, t->bitsize);
+
+    dump_val(t, v);
+}
+
+static void
+dump_thresh(uint8_t *sdr, struct sdr_field *t, int is_rel)
+{
+    int v = extract_bits(sdr, t);
+    int m, b, r_exp, b_exp, is_signed;
+    double fx;
+
+    is_signed = get_sdr_bits(sdr, 21, 6, 2);
+    if (is_signed)
+	v = sign_extend(v, t->bitsize);
+
+    m = get_sdr_bits(sdr, 25, 0, 8);
+    m |= get_sdr_bits(sdr, 26, 6, 2) << 8;
+    if (m & (1 << 9))
+	m |= (~0U << 10);
+    b = get_sdr_bits(sdr, 27, 0, 8);
+    b |= get_sdr_bits(sdr, 28, 6, 2) << 8;
+    if (b & (1 << 9))
+	b |= (~0U << 10);
+    r_exp = get_sdr_bits(sdr, 30, 4, 4);
+    if (r_exp & (1 << 3))
+	r_exp |= (~0U << 4);
+    b_exp = get_sdr_bits(sdr, 30, 0, 4);
+    if (b_exp & (1 << 3))
+	b_exp |= (~0U << 4);
+
+    fx = ((double) m) * ((double) v);
+    if (!is_rel)
+	fx += ((double) b) * pow(10, b_exp);
+    fx *= pow(10, r_exp);
+
+    fprintf(outfile, "%f\n", fx);
+}
+
+static void
+dump_string(uint8_t *sdr, struct sdr_field *t, unsigned int sdr_len)
+{
+    unsigned char *sptr = ((unsigned char *) sdr) + t->pos - 1;
+    char str[IPMI_MAX_STR_LEN + 1];
+    unsigned int out_len;
+    enum ipmi_str_type_e stype;
+    int err;
+
+    err = ipmi_get_device_string(&sptr, sdr_len + 5 - t->pos + 1, str,
+				 IPMI_STR_SDR_SEMANTICS, 0, &stype,
+				 sizeof(str), &out_len);
+    if (err) {
+	fprintf(stderr, "Warning: Invalid SDR string, search for **INVALID**"
+		" in output\n");
+	fprintf(outfile, "**INVALID**\n");
+	return;
+    }
+
+    str[out_len] = '\0';
+    fprintf(outfile, "%s\n", str);
+}
+
+static void
+dump_sdr_field(uint8_t *sdr, struct sdr_field *t, unsigned int len)
+{
+    if (t->type == SDR_MULTIBITS2)
+	return;
+
+    fprintf(outfile, "\t%s\t", t->name);
+    switch(t->type)
+    {
+    case SDR_BITS:
+    case SDR_SBITS:
+	dump_int(sdr, t, t->type == SDR_SBITS);
+	break;
+
+    case SDR_BOOLBIT:
+	if (sdr[t->pos] & (1 << t->bitoff))
+	    fprintf(outfile, "true\n");
+	else
+	    fprintf(outfile, "false\n");
+	break;
+
+    case SDR_MULTIBITS:
+    case SDR_MULTISBITS:
+	dump_multibits(sdr, t, t->type == SDR_MULTISBITS);
+	break;
+
+    case SDR_MULTIBITS2:
+	break;
+
+    case SDR_THRESH:
+    case SDR_THRESHREL:
+	dump_thresh(sdr, t, t->type == SDR_THRESHREL);
+	break;
+
+    case SDR_STRING:
+	dump_string(sdr, t, len);
+	break;
+    }
+}
+
+static void
+decompile_file(FILE *f)
+{
+    uint8_t sdr[261];
+    size_t l;
+    int recnum = 0;
+    struct sdr_field *t;
+    unsigned int tlen, sdr_len;
+    size_t offset = 0;
+    unsigned int i;
+
+    l = fread(sdr, 5, 1, f);
+    while (l == 1) {
+	uint16_t recid = sdr[0] | sdr[1] << 8;
+	int badtype = 0;
+
+	if (sdr[2] != 0x51) {
+	    fprintf(stderr, "SDR %d offset %ld: Unknown SDR version: 0x%x\n",
+		    recnum, offset, sdr[2]);
+	}
+
+	fprintf(outfile, "# Record id: %d\n", recid);
+	fprintf(outfile, "# Record num: %d\n", recnum);
+
+	if (get_sdr_type(sdr[3], &t, &tlen, &sdr_len, NULL)) {
+	    fprintf(outfile, "# unknown type: %d\n", sdr[3]);
+	    badtype = 1;
+	} else if (sdr[4] < sdr_len - 5) {
+	    fprintf(stderr, "SDR %d offset %ld type %d: not long enough: %d,"
+		    " expected %d\n",
+		    recnum, offset, sdr[3], sdr[4], sdr_len - 5);
+	    exit(1);
+	}
+	
+	l = fread(sdr + 5, sdr[4], 1, f);
+	if (l != 1) {
+	    if (ferror(f)) {
+		fprintf(stderr, "Error reading file: %s\n", strerror(errno));
+		exit(1);
+	    } else {
+		fprintf(stderr, "End of file in the middle of"
+			" record %d offset %ld\n", recnum, offset);
+		exit(1);
+	    }
+	}
+
+	if (!badtype) {
+	    fprintf(outfile, "sdr %d\n", sdr[3]);
+	    for (i = 0; i < tlen; i++)
+		dump_sdr_field(sdr, t + i, sdr[4]);
+	    fprintf(outfile, "endsdr\n\n");
+	}
+
+	offset += sdr[4] + 5;
+
+	l = fread(sdr, 5, 1, f);
+	recnum++;
+    }
+
+    if (ferror(f)) {
+	fprintf(stderr, "Error reading file: %s\n", strerror(errno));
+	exit(1);
+    }
+}
+
+static char *progname;
+
+static void
+help(void)
+{
+    fprintf(stderr, "%s [-r] [-o <outfile>] [-d] <input file>\n", progname);
+    exit(1);
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -1653,8 +1963,10 @@ main(int argc, char *argv[])
     unsigned int sdrnum = 1;
     int argn;
     int outraw = 0;
+    int decompile = 0;
 
     progname = argv[0];
+    outfile = stdout;
 
     for (argn = 1; argn < argc; argn++) {
 	if (argv[argn][0] != '-')
@@ -1663,6 +1975,15 @@ main(int argc, char *argv[])
 	    break;
 	if (strcmp(argv[argn], "-r") == 0) {
 	    outraw = 1;
+	} else if (strcmp(argv[argn], "-d") == 0) {
+	    decompile = 1;
+	} else if (strcmp(argv[argn], "-o") == 0) {
+	    argn++;
+	    if (argn == argc) {
+		fprintf(stderr, "No value supplied for -o\n");
+		exit(1);
+	    }
+	    outfname = argv[argn];
 	} else {
 	    fprintf(stderr, "Invalid option: %s\n", argv[argn]);
 	    exit(1);
@@ -1674,13 +1995,19 @@ main(int argc, char *argv[])
 	help();
     }
 
-    f = fopen(argv[1], "r");
+    f = fopen(argv[argn], "r");
     if (!f) {
-	fprintf(stderr, "Unable to open input file %s\n", argv[1]);
+	fprintf(stderr, "Unable to open input file %s\n", argv[argn]);
 	exit(1);
     }
 
-    if (!outraw) {
+    argn++;
+    if (argn < argc) {
+	fprintf(stderr, "Extra arguments at end: %s\n", argv[argn]);
+	exit(1);
+    }
+
+    if (!outraw && !decompile) {
 	p = alloc_persist("");
 	if (!p) {
 	    fprintf(stderr, "Out of memory\n");
@@ -1688,13 +2015,25 @@ main(int argc, char *argv[])
 	}
     }
 
-    parse_file(argv[1], f, p, outraw, &sdrnum);
+    if (outfname) {
+	outfile = fopen(outfname, "w");
+	if (!outfile) {
+	    fprintf(stderr, "Unable to open output file %s: %s\n",
+		    outfname, strerror(errno));
+	    exit(1);
+	}
+    }
+
+    if (decompile)
+	decompile_file(f);
+    else
+	compile_file(argv[argn], f, p, outraw, &sdrnum);
 
     fclose(f);
 
-    if (!outraw) {
+    if (!outraw && !decompile) {
 	add_persist_int(p, time(NULL), "last_add_time");
-	write_persist_file(p, stdout);
+	write_persist_file(p, outfile);
 	free_persist(p);
     }
 
