@@ -745,7 +745,8 @@ tm_setup(serserv_data_t *si)
 #define   VM_CAPABILITIES_IRQ	0x04
 #define   VM_CAPABILITIES_NMI	0x08
 #define   VM_CAPABILITIES_ATTN	0x10
-#define VM_CMD_FORCEOFF		0x09
+#define   VM_CAPABILITIES_GRACEFUL_SHUTDOWN 0x20
+#define VM_CMD_GRACEFUL_SHUTDOWN 0x09
 
 struct vm_data {
     unsigned char recv_msg[IPMI_SIM_MAX_MSG_LENGTH + 4];
@@ -806,6 +807,8 @@ vm_handle_cmd(unsigned char *imsg, unsigned int len, serserv_data_t *si)
 	    return;
 	if (imsg[1] & VM_CAPABILITIES_POWER)
 	    si->channel.hw_capabilities |= (1 << HW_OP_POWERON);
+	if (imsg[1] & VM_CAPABILITIES_GRACEFUL_SHUTDOWN)
+	    si->channel.hw_capabilities |= (1 << HW_OP_GRACEFUL_SHUTDOWN);
 	if (imsg[1] & VM_CAPABILITIES_RESET)
 	    si->channel.hw_capabilities |= (1 << HW_OP_RESET);
 	if (imsg[1] & VM_CAPABILITIES_IRQ)
@@ -961,9 +964,9 @@ vm_hw_op(channel_t *chan, unsigned int op)
 	    chan->stop_cmd(chan, !si->connected);
 	break;
 	
-    case HW_OP_FORCEOFF:
+    case HW_OP_GRACEFUL_SHUTDOWN:
 	if (si->connected)
-	    vm_add_char(VM_CMD_FORCEOFF, c, &len);
+	    vm_add_char(VM_CMD_GRACEFUL_SHUTDOWN, c, &len);
 	break;
 	
     case HW_OP_SEND_NMI:
@@ -1003,6 +1006,7 @@ vm_connected(serserv_data_t *si)
     c[len++] = VM_CMD_CHAR;
     raw_send(si, c, len);
     si->connected = 1;
+    ipmi_resend_atn(&si->channel);
 }
 
 static void
@@ -1255,7 +1259,7 @@ serserv_read_config(char **tokptr, sys_data_t *sys, const char **errstr)
     err = get_sock_addr(tokptr, &ser->addr.addr, &ser->addr.addr_len,
 			NULL, SOCK_STREAM, errstr);
     if (err)
-	return err;
+	goto out_err;
 
     for (tok = mystrtok(NULL, " \t\n", tokptr); tok;
 	 tok = mystrtok(NULL, " \t\n", tokptr)) {
