@@ -429,6 +429,7 @@ int init_oem_test(void);
 int i_ipmi_smi_init(os_handler_t *os_hnd);
 int i_ipmi_lan_init(os_handler_t *os_hnd);
 int ipmi_malloc_init(os_handler_t *os_hnd);
+void ipmi_malloc_shutdown(void);
 int i_ipmi_rakp_init(void);
 int i_ipmi_aes_cbc_init(void);
 int i_ipmi_hmac_init(void);
@@ -576,6 +577,8 @@ ipmi_shutdown(void)
 	ipmi_os_handler->destroy_lock(ipmi_os_handler, seq_lock);
     if (con_type_list)
 	locked_list_destroy(con_type_list);
+    ipmi_debug_malloc_cleanup();
+    ipmi_malloc_shutdown();
 
     ipmi_os_handler = NULL;
 
