@@ -521,23 +521,17 @@ find_sol_connection(ipmi_sol_conn_t *sol)
     return NULL;
 }
 
+static void ipmid_changed(ipmi_con_t   *ipmid,
+			  int          err,
+			  unsigned int port_num,
+			  int          any_port_up,
+			  void         *cb_data);
 
 static void
-sol_cleanup(ipmi_sol_conn_t *conn)
+sol_connection_closed(ipmi_con_t *ipmi, void *cb_data)
 {
-    if (conn->state != ipmi_sol_state_closed)
-	ipmi_sol_force_close(conn);
+    ipmi_sol_conn_t *conn = cb_data;
 
-    delete_connection(conn);
-
-    while (conn->waiting_packets) {
-	sol_in_packet_info_t *to_free = conn->waiting_packets;
-
-	conn->waiting_packets = to_free->next;
-	ipmi_mem_free(to_free);
-    }
-
-    conn->ipmi->close_connection(conn->ipmi);
     if (conn->transmitter.packet_lock)
 	ipmi_destroy_lock(conn->transmitter.packet_lock);
     if (conn->transmitter.queue_lock)
@@ -555,6 +549,30 @@ sol_cleanup(ipmi_sol_conn_t *conn)
     ipmi_mem_free(conn);
 }
 
+static void
+sol_cleanup(ipmi_sol_conn_t *conn)
+{
+    if (conn->ipmid)
+	conn->ipmid->remove_con_change_handler(conn->ipmid, ipmid_changed,
+					       conn);
+
+    if (conn->state != ipmi_sol_state_closed)
+	ipmi_sol_force_close(conn);
+
+    delete_connection(conn);
+
+    while (conn->waiting_packets) {
+	sol_in_packet_info_t *to_free = conn->waiting_packets;
+
+	conn->waiting_packets = to_free->next;
+	ipmi_mem_free(to_free);
+    }
+
+    if (conn->ipmi->close_connection_done(conn->ipmid, sol_connection_closed,
+					  conn))
+	sol_connection_closed(NULL, conn);
+}
+
 /**
  * Tell the system that a user is done with the connection.
  */
@@ -1747,10 +1765,14 @@ dequeue_head(ipmi_sol_transmitter_context_t *transmitter, int error)
     qitem = transmitter->outgoing_queue.head;
 
     if (qitem) {
-	if (qitem->transmit_complete_callback)
+	if (qitem->transmit_complete_callback) {
+	    ipmi_unlock(transmitter->queue_lock);
+	    ipmi_unlock(transmitter->packet_lock);
 	    (qitem->transmit_complete_callback)(transmitter->sol_conn,
 						error, qitem->cb_data);
-	
+	    ipmi_lock(transmitter->packet_lock);
+	    ipmi_lock(transmitter->queue_lock);
+	}
 	if (qitem->data)
 	    ipmi_mem_free(qitem->data);
 	
@@ -1794,6 +1816,8 @@ transmitter_handle_acknowledge(ipmi_sol_conn_t *conn,
 			       int             error,
 			       int             acknowledged_char_count)
 {
+    ipmi_sol_outgoing_queue_item_t *qitem;
+
     /*
      * Handle the in-band data by iterating through packets, and:
      *	1) Counting off how many bytes of this packet have been ACKed,
@@ -1805,8 +1829,6 @@ transmitter_handle_acknowledge(ipmi_sol_conn_t *conn,
 	     acknowledged_char_count);
 #endif
 
-    ipmi_sol_outgoing_queue_item_t *qitem;
-
     do {
 	int avail_this_pkt;
 	int this_ack;
@@ -2003,6 +2025,7 @@ ipmi_sol_write(ipmi_sol_conn_t               *conn,
 	       void                          *cb_data)
 {
     int rv;
+
     if (count <= 0)
 	return EINVAL;
 
@@ -2012,8 +2035,9 @@ ipmi_sol_write(ipmi_sol_conn_t               *conn,
     {
 	rv = add_to_transmit_queue(&conn->transmitter, buf, count, 0,
 				   cb, cb_data);
-    } else
+    } else {
 	rv = EINVAL;
+    }
     ipmi_unlock(conn->transmitter.packet_lock);
     return rv;
 }
@@ -2533,7 +2557,7 @@ finish_activate_payload(ipmi_sol_conn_t *conn)
     ipmi_sol_set_connection_state(conn, ipmi_sol_state_connected, 0);
 }
 
-static void ipmid_changed(ipmi_con_t   *ipmid, 
+static void ipmid_changed(ipmi_con_t   *ipmid,
 			  int          err,
 			  unsigned int port_num,
 			  int          any_port_up,
@@ -2541,6 +2565,7 @@ static void ipmid_changed(ipmi_con_t   *ipmid,
 {
     ipmi_sol_conn_t *conn = cb_data;
 
+    ipmi_lock(conn->transmitter.packet_lock);
     if (err) {
 	ipmi_log(IPMI_LOG_SEVERE,
 		 "ipmi_sol.c(handle_active_payload_response): "
@@ -2549,11 +2574,13 @@ static void ipmid_changed(ipmi_con_t   *ipmid,
     }
 
     finish_activate_payload(conn);
+    ipmi_unlock(conn->transmitter.packet_lock);
     return;
 
  out_err:
     send_close(conn, NULL); 
     ipmi_sol_set_connection_state(conn, ipmi_sol_state_closed, err);
+    ipmi_unlock(conn->transmitter.packet_lock);
 }
 
 /*
@@ -2604,6 +2631,7 @@ setup_new_ipmi(ipmi_sol_conn_t *conn)
 		 "Error setting up new connection: %d.", rv);
 	return rv;
     }
+    ipmi_free_args(args);
 
     rv = conn->ipmid->add_con_change_handler(conn->ipmid, ipmid_changed, conn);
     if (rv) {
@@ -3199,8 +3227,6 @@ handle_deactivate_payload_response(ipmi_sol_conn_t *conn,
 	    ipmi_sol_set_connection_state(conn, ipmi_sol_state_closed, 0);
 
     transmitter_shutdown(&conn->transmitter, 0);
-
-    return;
 }
 
 int
@@ -3221,7 +3247,7 @@ ipmi_sol_close(ipmi_sol_conn_t *conn)
 
 
 int
-ipmi_sol_force_close(ipmi_sol_conn_t *conn)
+ipmi_sol_force_close_wsend(ipmi_sol_conn_t *conn, int rem_close)
 {
     ipmi_lock(conn->transmitter.packet_lock);
     if (conn->state == ipmi_sol_state_closed) {
@@ -3229,7 +3255,7 @@ ipmi_sol_force_close(ipmi_sol_conn_t *conn)
 	return EINVAL;
     }
 
-    if (conn->state != ipmi_sol_state_closing)
+    if (rem_close && conn->state != ipmi_sol_state_closing)
 	/*
 	 * Try to be polite to the BMC. Don't ask for a callback,
 	 * cos we'll be gone!
@@ -3246,6 +3272,11 @@ ipmi_sol_force_close(ipmi_sol_conn_t *conn)
     return 0;
 }
 
+int
+ipmi_sol_force_close(ipmi_sol_conn_t *conn)
+{
+    return ipmi_sol_force_close_wsend(conn, 1);
+}
 
 int
 ipmi_sol_free(ipmi_sol_conn_t *conn)
