@@ -52,6 +52,7 @@
 #include <syslog.h>
 #include <signal.h>
 #include <string.h>
+#include <assert.h>
 #ifdef HAVE_EPOLL_PWAIT
 #include <sys/epoll.h>
 #else
@@ -60,11 +61,23 @@
 #define EPOLL_CTL_MOD 0
 #endif
 
+struct sel_runner_s
+{
+    struct selector_s *sel;
+    sel_runner_func_t func;
+    void *cb_data;
+    int in_use;
+    sel_runner_t *next;
+};
+
 typedef struct fd_state_s
 {
     int               deleted;
     unsigned int      use_count;
     sel_fd_cleared_cb done;
+    sel_runner_t      done_runner;
+    int               tmp_fd;
+    void              *done_cbdata;
 } fd_state_t;
 
 /* The control structure for each file descriptor. */
@@ -78,6 +91,9 @@ typedef struct fd_control_s
     sel_fd_handler_t handle_read;
     sel_fd_handler_t handle_write;
     sel_fd_handler_t handle_except;
+#ifdef HAVE_EPOLL_PWAIT
+    uint32_t saved_events;
+#endif
 } fd_control_t;
 
 typedef struct heap_val_s
@@ -147,7 +163,7 @@ heap_cmp_key(heap_val_t *v1, heap_val_t *v2)
 
 /* Used to build a list of threads that may need to be woken if a
    timer on the top of the heap changes, or an FD is added/removed.
-   See wake_sel_thread() for more info. */
+   See i_wake_sel_thread() for more info. */
 typedef struct sel_wait_list_s
 {
     /* The thread to wake up. */
@@ -157,22 +173,9 @@ typedef struct sel_wait_list_s
     sel_send_sig_cb send_sig;
     void            *send_sig_cb_data;
 
-    /* This is the memory used to hold the timeout for select
-       operation. */
-    volatile struct timeval *timeout;
-
     struct sel_wait_list_s *next, *prev;
 } sel_wait_list_t;
 
-struct sel_runner_s
-{
-    struct selector_s *sel;
-    sel_runner_func_t func;
-    void *cb_data;
-    int in_use;
-    sel_runner_t *next;
-};
-
 struct selector_s
 {
     /* This is an array of all the file descriptors possible.  This is
@@ -189,13 +192,18 @@ struct selector_s
     volatile int maxfd; /* The largest file descriptor registered with
 			   this code. */
 
+    /* If something is deleted, we increment this count.  This way when
+       a select/epoll returns a non-timeout, we know that we need to ignore
+       it as it may be  from the just deleted fd. */
+    unsigned long fd_del_count;
+
     void *fd_lock;
 
     /* The timer heap. */
     theap_t timer_heap;
 
     /* This is a list of items waiting to be woken up because they are
-       sitting in a select.  See wake_sel_thread() for more info. */
+       sitting in a select.  See i_wake_sel_thread() for more info. */
     sel_wait_list_t wait_list;
 
     void *timer_lock;
@@ -255,25 +263,24 @@ sel_fd_unlock(struct selector_s *sel)
    this after we have calculated the timeout, but before we have
    called select, thus only things in the wait list matter. */
 static void
-wake_sel_thread(struct selector_s *sel)
+i_wake_sel_thread(struct selector_s *sel)
 {
     sel_wait_list_t *item;
 
     item = sel->wait_list.next;
     while (item != &sel->wait_list) {
-	item->timeout->tv_sec = 0;
-	item->timeout->tv_usec = 0;
 	if (item->send_sig)
 	    item->send_sig(item->thread_id, item->send_sig_cb_data);
 	item = item->next;
     }
 }
 
-static void
-wake_fd_sel_thread(struct selector_s *sel)
+void
+sel_wake_all(struct selector_s *sel)
 {
-    wake_sel_thread(sel);
-    sel_fd_unlock(sel);
+    sel_timer_lock(sel);
+    i_wake_sel_thread(sel);
+    sel_timer_unlock(sel);
 }
 
 static void
@@ -281,7 +288,7 @@ wake_timer_sel_thread(struct selector_s *sel, volatile sel_timer_t *old_top)
 {
     if (old_top != theap_get_top(&sel->timer_heap))
 	/* If the top value changed, restart the waiting thread. */
-	wake_sel_thread(sel);
+	i_wake_sel_thread(sel);
 }
 
 /* Wait list management.  These *must* be called with the timer list
@@ -291,10 +298,9 @@ static void
 add_sel_wait_list(struct selector_s *sel, sel_wait_list_t *item,
 		  sel_send_sig_cb send_sig,
 		  void            *cb_data,
-		  long thread_id, volatile struct timeval *timeout)
+		  long thread_id)
 {
     item->thread_id = thread_id;
-    item->timeout = timeout;
     item->send_sig = send_sig;
     item->send_sig_cb_data = cb_data;
     item->next = sel->wait_list.next;
@@ -322,9 +328,11 @@ init_fd(fd_control_t *fd)
 
 #ifdef HAVE_EPOLL_PWAIT
 static int
-sel_update_epoll(struct selector_s *sel, int fd, int op)
+sel_update_fd(struct selector_s *sel, int fd, int op)
 {
+    fd_control_t *fdc = (fd_control_t *) &sel->fds[fd];
     struct epoll_event event;
+    int rv;
 
     if (sel->epollfd < 0)
 	return 1;
@@ -332,24 +340,52 @@ sel_update_epoll(struct selector_s *sel, int fd, int op)
     memset(&event, 0, sizeof(event));
     event.events = EPOLLONESHOT;
     event.data.fd = fd;
-    if (FD_ISSET(fd, &sel->read_set))
-	event.events |= EPOLLIN | EPOLLHUP;
-    if (FD_ISSET(fd, &sel->write_set))
-	event.events |= EPOLLOUT;
-    if (FD_ISSET(fd, &sel->write_set))
-	event.events |= EPOLLERR | EPOLLPRI;
-
-    epoll_ctl(sel->epollfd, op, fd, &event);
+    if (fdc->saved_events) {
+	if (op == EPOLL_CTL_DEL)
+	    return 0;
+	if (!FD_ISSET(fd, &sel->read_set) && !FD_ISSET(fd, &sel->except_set))
+	    return 0;
+	fdc->saved_events = 0;
+	op = EPOLL_CTL_ADD;
+	if (FD_ISSET(fd, &sel->read_set))
+	    event.events |= EPOLLIN | EPOLLHUP;
+	if (FD_ISSET(fd, &sel->except_set))
+	    event.events |= EPOLLERR | EPOLLPRI;
+    } else if (op != EPOLL_CTL_DEL) {
+	if (FD_ISSET(fd, &sel->read_set))
+	    event.events |= EPOLLIN | EPOLLHUP;
+	if (FD_ISSET(fd, &sel->write_set))
+	    event.events |= EPOLLOUT;
+	if (FD_ISSET(fd, &sel->except_set))
+	    event.events |= EPOLLERR | EPOLLPRI;
+    }
+    /* This should only fail due to system problems, and if that's the case,
+       well, we should probably terminate. */
+    rv = epoll_ctl(sel->epollfd, op, fd, &event);
+    if (rv) {
+	perror("epoll_ctl");
+	assert(0);
+    }
     return 0;
 }
 #else
 static int
-sel_update_epoll(struct selector_s *sel, int fd, int op)
+sel_update_fd(struct selector_s *sel, int fd, int op)
 {
     return 1;
 }
 #endif
 
+static void
+finish_oldstate(sel_runner_t *runner, void *cbdata)
+{
+    fd_state_t *oldstate = cbdata;
+
+    if (oldstate->done)
+	oldstate->done(oldstate->tmp_fd, oldstate->done_cbdata);
+    free(oldstate);
+}
+
 /* Set the handlers for a file descriptor. */
 int
 sel_set_fd_handlers(struct selector_s *sel,
@@ -368,9 +404,10 @@ sel_set_fd_handlers(struct selector_s *sel,
     state = malloc(sizeof(*state));
     if (!state)
 	return ENOMEM;
-    state->deleted = 0;
-    state->use_count = 0;
+    memset(state, 0, sizeof(*state));
     state->done = done;
+    memset(&state->done_runner, 0, sizeof(state->done_runner));
+    state->done_runner.sel = sel;
 
     sel_fd_lock(sel);
     fdc = (fd_control_t *) &(sel->fds[fd]);
@@ -378,6 +415,10 @@ sel_set_fd_handlers(struct selector_s *sel,
 	oldstate = fdc->state;
 	olddata = fdc->data;
 	added = 0;
+#ifdef HAVE_EPOLL_PWAIT
+	fdc->saved_events = 0;
+#endif
+	sel->fd_del_count++;
     }
     fdc->state = state;
     fdc->data = data;
@@ -391,30 +432,27 @@ sel_set_fd_handlers(struct selector_s *sel,
 	    sel->maxfd = fd;
 	}
 
-	if (sel_update_epoll(sel, fd, EPOLL_CTL_ADD)) {
-	    wake_fd_sel_thread(sel);
-	    goto out;
-	}
+	if (sel_update_fd(sel, fd, EPOLL_CTL_ADD))
+	    sel_wake_all(sel);
+    } else {
+	if (sel_update_fd(sel, fd, EPOLL_CTL_MOD))
+	    sel_wake_all(sel);
     }
     sel_fd_unlock(sel);
 
- out:
     if (oldstate) {
 	oldstate->deleted = 1;
 	if (oldstate->use_count == 0) {
-	    if (oldstate->done)
-		oldstate->done(fd, olddata);
-	    free(oldstate);
+	    oldstate->tmp_fd = fd;
+	    oldstate->done_cbdata = olddata;
+	    sel_run(&oldstate->done_runner, finish_oldstate, oldstate);
 	}
     }
     return 0;
 }
 
-/* Clear the handlers for a file descriptor and remove it from
-   select's monitoring. */
-void
-sel_clear_fd_handlers(struct selector_s *sel,
-		      int        fd)
+static void
+i_sel_clear_fd_handler(struct selector_s *sel, int fd, int imm)
 {
     fd_control_t *fdc;
     fd_state_t   *oldstate = NULL;
@@ -428,7 +466,11 @@ sel_clear_fd_handlers(struct selector_s *sel,
 	olddata = fdc->data;
 	fdc->state = NULL;
 
-	sel_update_epoll(sel, fd, EPOLL_CTL_DEL);
+	sel_update_fd(sel, fd, EPOLL_CTL_DEL);
+#ifdef HAVE_EPOLL_PWAIT
+	fdc->saved_events = 0;
+#endif
+	sel->fd_del_count++;
     }
 
     init_fd(fdc);
@@ -447,14 +489,34 @@ sel_clear_fd_handlers(struct selector_s *sel,
 
     if (oldstate) {
 	oldstate->deleted = 1;
-	if (oldstate->use_count == 0) {
-	    if (oldstate->done)
-		oldstate->done(fd, olddata);
-	    free(oldstate);
+	if (imm) {
+	    assert(oldstate->use_count == 0);
+	} else if (oldstate->use_count == 0) {
+	    oldstate->tmp_fd = fd;
+	    oldstate->done_cbdata = olddata;
+	    sel_run(&oldstate->done_runner, finish_oldstate, oldstate);
 	}
     }
 }
 
+/* Clear the handlers for a file descriptor and remove it from
+   select's monitoring. */
+void
+sel_clear_fd_handlers(struct selector_s *sel, int fd)
+{
+    i_sel_clear_fd_handler(sel, fd, 0);
+}
+
+/* Clear the handlers for a file descriptor and remove it from
+   select's monitoring, except this can only be called if no
+   callbacks are active (like you haven't started listening
+   yet).  If a callback is active, it will assert. */
+void
+sel_clear_fd_handlers_imm(struct selector_s *sel, int fd)
+{
+    i_sel_clear_fd_handler(sel, fd, 1);
+}
+
 /* Set whether the file descriptor will be monitored for data ready to
    read on the file descriptor. */
 void
@@ -475,10 +537,8 @@ sel_set_fd_read_handler(struct selector_s *sel, int fd, int state)
 	    goto out;
 	FD_CLR(fd, &sel->read_set);
     }
-    if (sel_update_epoll(sel, fd, EPOLL_CTL_MOD)) {
-	wake_fd_sel_thread(sel);
-	return;
-    }
+    if (sel_update_fd(sel, fd, EPOLL_CTL_MOD))
+	sel_wake_all(sel);
 
  out:
     sel_fd_unlock(sel);
@@ -504,10 +564,8 @@ sel_set_fd_write_handler(struct selector_s *sel, int fd, int state)
 	    goto out;
 	FD_CLR(fd, &sel->write_set);
     }
-    if (sel_update_epoll(sel, fd, EPOLL_CTL_MOD)) {
-	wake_fd_sel_thread(sel);
-	return;
-    }
+    if (sel_update_fd(sel, fd, EPOLL_CTL_MOD))
+	sel_wake_all(sel);
 
  out:
     sel_fd_unlock(sel);
@@ -533,10 +591,8 @@ sel_set_fd_except_handler(struct selector_s *sel, int fd, int state)
 	    goto out;
 	FD_CLR(fd, &sel->except_set);
     }
-    if (sel_update_epoll(sel, fd, EPOLL_CTL_MOD)) {
-	wake_fd_sel_thread(sel);
-	return;
-    }
+    if (sel_update_fd(sel, fd, EPOLL_CTL_MOD))
+	sel_wake_all(sel);
 
  out:
     sel_fd_unlock(sel);
@@ -566,6 +622,19 @@ diff_timeval(struct timeval *dest,
     }
 }
 
+static void
+add_timeval(struct timeval *dest,
+	    struct timeval *left,
+	    struct timeval *right)
+{
+    dest->tv_sec = left->tv_sec + right->tv_sec;
+    dest->tv_usec = left->tv_usec + right->tv_usec;
+    while (dest->tv_usec > 1000000) {
+	dest->tv_usec -= 1000000;
+	dest->tv_sec++;
+    }
+}
+
 int
 sel_alloc_timer(struct selector_s     *sel,
 		sel_timeout_handler_t handler,
@@ -588,6 +657,24 @@ sel_alloc_timer(struct selector_s     *sel,
     return 0;
 }
 
+static int
+sel_stop_timer_i(struct selector_s *sel, sel_timer_t *timer)
+{
+    if (timer->val.stopped)
+	return ETIMEDOUT;
+
+    if (timer->val.in_heap) {
+	volatile sel_timer_t *old_top = theap_get_top(&sel->timer_heap);
+
+	theap_remove(&sel->timer_heap, timer);
+	timer->val.in_heap = 0;
+	wake_timer_sel_thread(sel, old_top);
+    }
+    timer->val.stopped = 1;
+
+    return 0;
+}
+
 int
 sel_free_timer(sel_timer_t *timer)
 {
@@ -595,9 +682,8 @@ sel_free_timer(sel_timer_t *timer)
     int in_handler;
 
     sel_timer_lock(sel);
-    if (timer->val.in_heap) {
-	sel_stop_timer(timer);
-    }
+    if (timer->val.in_heap)
+	sel_stop_timer_i(sel, timer);
     timer->val.freed = 1;
     in_handler = timer->val.in_handler;
     sel_timer_unlock(sel);
@@ -613,7 +699,7 @@ sel_start_timer(sel_timer_t    *timer,
 		struct timeval *timeout)
 {
     struct selector_s *sel = timer->val.sel;
-    volatile sel_timer_t *top;
+    volatile sel_timer_t *old_top;
 
     sel_timer_lock(sel);
     if (timer->val.in_heap) {
@@ -621,7 +707,7 @@ sel_start_timer(sel_timer_t    *timer,
 	return EBUSY;
     }
 
-    top = theap_get_top(&sel->timer_heap);
+    old_top = theap_get_top(&sel->timer_heap);
 
     timer->val.timeout = *timeout;
 
@@ -632,7 +718,7 @@ sel_start_timer(sel_timer_t    *timer,
     }
     timer->val.stopped = 0;
 
-    wake_timer_sel_thread(sel, top);
+    wake_timer_sel_thread(sel, old_top);
 
     sel_timer_unlock(sel);
 
@@ -643,26 +729,13 @@ int
 sel_stop_timer(sel_timer_t *timer)
 {
     struct selector_s *sel = timer->val.sel;
-    volatile sel_timer_t *top;
+    int rv;
 
     sel_timer_lock(sel);
-    if (timer->val.stopped) {
-	sel_timer_unlock(sel);
-	return ETIMEDOUT;
-    }
-
-    if (timer->val.in_heap) {
-	top = theap_get_top(&sel->timer_heap);
-
-	theap_remove(&sel->timer_heap, timer);
-	timer->val.in_heap = 0;
-	wake_timer_sel_thread(sel, top);
-    }
-    timer->val.stopped = 1;
-
+    rv = sel_stop_timer_i(sel, timer);
     sel_timer_unlock(sel);
 
-    return 0;
+    return rv;
 }
 
 int
@@ -671,34 +744,41 @@ sel_stop_timer_with_done(sel_timer_t *timer,
 			 void *cb_data)
 {
     struct selector_s *sel = timer->val.sel;
-    volatile sel_timer_t *top;
 
     sel_timer_lock(sel);
+    if (timer->val.done_handler) {
+	sel_timer_unlock(sel);
+	return EBUSY;
+    }
     if (timer->val.stopped) {
 	sel_timer_unlock(sel);
-	goto out;
+	return ETIMEDOUT;
     }
 
-    if (timer->val.in_handler) {
-	timer->val.done_handler = done_handler;
-	timer->val.done_cb_data = cb_data;
-	sel_timer_unlock(sel);
-	return 0;
-    }
+    timer->val.stopped = 1;
 
+    timer->val.done_handler = done_handler;
+    timer->val.done_cb_data = cb_data;
+
+    if (timer->val.in_handler)
+	goto out_unlock;
+
+    /*
+     * We don't want to run the done handler here do avoid locking
+     * issues.  So set it in_handler and stick it on the top of the
+     * heap with an immediate timeout so it will be processed now.
+     */
+    timer->val.in_handler = 1;
     if (timer->val.in_heap) {
-	top = theap_get_top(&sel->timer_heap);
-
 	theap_remove(&sel->timer_heap, timer);
 	timer->val.in_heap = 0;
-
-	wake_timer_sel_thread(sel, top);
     }
-    timer->val.stopped = 1;
-    sel_timer_unlock(sel);
+    sel_get_monotonic_time(&timer->val.timeout);
+    theap_add(&sel->timer_heap, timer);
+    wake_timer_sel_thread(sel, NULL);
 
- out:
-    done_handler(sel, timer, cb_data);
+ out_unlock:
+    sel_timer_unlock(sel);
     return 0;
 }
 
@@ -720,32 +800,42 @@ sel_get_monotonic_time(struct timeval *tv)
  */
 static void
 process_timers(struct selector_s       *sel,
+	       unsigned int            *count,
 	       volatile struct timeval *timeout)
 {
     struct timeval now;
     sel_timer_t    *timer;
-    int            called = 0;
 
     timer = theap_get_top(&sel->timer_heap);
     sel_get_monotonic_time(&now);
     while (timer && cmp_timeval(&now, &timer->val.timeout) >= 0) {
-	called = 1;
 	theap_remove(&(sel->timer_heap), timer);
 	timer->val.in_heap = 0;
 	timer->val.stopped = 1;
-	timer->val.in_handler = 1;
-	sel_timer_unlock(sel);
-	timer->val.handler(sel, timer, timer->val.user_data);
-	sel_timer_lock(sel);
-	timer->val.in_handler = 0;
+
+	/*
+	 * A timer may be in a handler here if it has been stopped with
+	 * a done_handler.  In that case the timer was stopped, so we
+	 * don't call the main handler.
+	 */
+	if (!timer->val.in_handler) {
+	    timer->val.in_handler = 1;
+	    sel_timer_unlock(sel);
+	    timer->val.handler(sel, timer, timer->val.user_data);
+	    sel_timer_lock(sel);
+	}
+	(*count)++;
 	if (timer->val.done_handler) {
 	    sel_timeout_handler_t done_handler = timer->val.done_handler;
 	    void *done_cb_data = timer->val.done_cb_data;
+
 	    timer->val.done_handler = NULL;
+	    timer->val.in_handler = 1;
 	    sel_timer_unlock(sel);
 	    done_handler(sel, timer, done_cb_data);
 	    sel_timer_lock(sel);
 	}
+	timer->val.in_handler = 0;
 	if (timer->val.freed)
 	    free(timer);
 	else if (!timer->val.stopped) {
@@ -757,7 +847,7 @@ process_timers(struct selector_s       *sel,
 	timer = theap_get_top(&sel->timer_heap);
     }
 
-    if (called) {
+    if (*count) {
 	/* If called, set the timeout to zero. */
 	timeout->tv_sec = 0;
 	timeout->tv_usec = 0;
@@ -829,9 +919,11 @@ sel_run(sel_runner_t *runner, sel_runner_func_t func, void *cb_data)
     return 0;
 }
 
-static void
+static unsigned int
 process_runners(struct selector_s *sel)
 {
+    int count = 0;
+
     while (sel->runner_head) {
 	sel_runner_t *runner = sel->runner_head;
 	sel_runner_func_t func;
@@ -845,8 +937,11 @@ process_runners(struct selector_s *sel)
 	cb_data = runner->cb_data;
 	sel_timer_unlock(sel);
 	func(runner, cb_data);
+	count++;
 	sel_timer_lock(sel);
     }
+
+    return count;
 }
 
 static void
@@ -884,6 +979,20 @@ handle_selector_call(struct selector_s *sel, int i, volatile fd_set *fdset,
     }
 }
 
+static void
+setup_my_sigmask(sigset_t *sigmask, sigset_t *isigmask)
+{
+    if (isigmask) {
+	*sigmask = *isigmask;
+    } else {
+#ifdef USE_PTHREADS
+	pthread_sigmask(SIG_SETMASK, NULL, sigmask);
+#else
+	sigprocmask(SIG_SETMASK, NULL, sigmask);
+#endif
+    }
+}
+
 /*
  * return == 0  when timeout
  * 	  >  0  when successful
@@ -891,7 +1000,8 @@ handle_selector_call(struct selector_s *sel, int i, volatile fd_set *fdset,
  */
 static int
 process_fds(struct selector_s	    *sel,
-	    volatile struct timeval *timeout)
+	    volatile struct timeval *timeout,
+	    sigset_t *isigmask)
 {
     fd_set      tmp_read_set;
     fd_set      tmp_write_set;
@@ -899,7 +1009,13 @@ process_fds(struct selector_s	    *sel,
     int i;
     int err;
     int num_fds;
+    sigset_t sigmask;
+    struct timespec ts = { .tv_sec = timeout->tv_sec,
+			   .tv_nsec = timeout->tv_usec * 1000 };
+    unsigned long entry_fd_del_count = sel->fd_del_count;
 
+    setup_my_sigmask(&sigmask, isigmask);
+ retry:
     sel_fd_lock(sel);
     memcpy(&tmp_read_set, (void *) &sel->read_set, sizeof(tmp_read_set));
     memcpy(&tmp_write_set, (void *) &sel->write_set, sizeof(tmp_write_set));
@@ -907,16 +1023,25 @@ process_fds(struct selector_s	    *sel,
     num_fds = sel->maxfd+1;
     sel_fd_unlock(sel);
 
-    err = select(num_fds,
-		 &tmp_read_set,
-		 &tmp_write_set,
-		 &tmp_except_set,
-		 (struct timeval *) timeout);
-    if (err <= 0)
+    sigdelset(&sigmask, sel->wake_sig);
+    err = pselect(num_fds,
+		  &tmp_read_set,
+		  &tmp_write_set,
+		  &tmp_except_set,
+		  &ts, &sigmask);
+    if (err < 0) {
+	if (errno == EBADF || errno == EBADFD)
+	    /* We raced, just retry it. */
+	    goto retry;
 	goto out;
+    }
 
     /* We got some I/O. */
     sel_fd_lock(sel);
+    if (entry_fd_del_count != sel->fd_del_count)
+	/* Something was deleted from the FD set, don't process this as it
+	   may be from the old fd wakeup. */
+	goto out_unlock;
     for (i = 0; i <= sel->maxfd; i++) {
 	if (FD_ISSET(i, &tmp_read_set))
 	    handle_selector_call(sel, i, &sel->read_set,
@@ -928,6 +1053,7 @@ process_fds(struct selector_s	    *sel,
 	    handle_selector_call(sel, i, &sel->except_set,
 				 sel->fds[i].handle_except);
     }
+ out_unlock:
     sel_fd_unlock(sel);
 out:
     return err;
@@ -935,12 +1061,17 @@ out:
 
 #ifdef HAVE_EPOLL_PWAIT
 static int
-process_fds_epoll(struct selector_s *sel, struct timeval *tvtimeout)
+process_fds_epoll(struct selector_s *sel, struct timeval *tvtimeout,
+		  sigset_t *isigmask)
 {
     int rv, fd;
     struct epoll_event event;
     int timeout;
     sigset_t sigmask;
+    fd_control_t *fdc;
+    unsigned long entry_fd_del_count = sel->fd_del_count;
+
+    setup_my_sigmask(&sigmask, isigmask);
 
     if (tvtimeout->tv_sec > 600)
 	 /* Don't wait over 10 minutes, to work around an old epoll bug
@@ -951,38 +1082,164 @@ process_fds_epoll(struct selector_s *sel, struct timeval *tvtimeout)
 	timeout = ((tvtimeout->tv_sec * 1000) +
 		   (tvtimeout->tv_usec + 999) / 1000);
 
-#ifdef USE_PTHREADS
-    pthread_sigmask(SIG_SETMASK, NULL, &sigmask);
-#else
-    sigprocmask(SIG_SETMASK, NULL, &sigmask);
-#endif
     sigdelset(&sigmask, sel->wake_sig);
     rv = epoll_pwait(sel->epollfd, &event, 1, timeout, &sigmask);
-
     if (rv <= 0)
 	return rv;
 
     sel_fd_lock(sel);
     fd = event.data.fd;
+    fdc = (fd_control_t *) &sel->fds[fd];
+    if (entry_fd_del_count != sel->fd_del_count)
+	/* Something was deleted from the FD set, don't process this as it
+	   may be from the old fd wakeup. */
+	goto rearm;
+    if (event.events & (EPOLLHUP | EPOLLERR)) {
+	/*
+	 * The crazy people that designed epoll made it so that EPOLLHUP
+	 * and EPOLLERR always wake it up, even if they are not set.  That
+	 * makes this fairly inconvenient, because we don't want to wake
+	 * up in that case unless we explicitly ask for it.  Fortunately,
+	 * in those cases we can pretty easily simulate it by just deleting
+	 * it, since in those cases you will not get anything but an
+	 * EPOLLHUP or EPOLLERR, anyway, and then doing the callback
+	 * by hand.
+	 */
+	sel_update_fd(sel, fd, EPOLL_CTL_DEL);
+	fdc->saved_events = event.events & (EPOLLHUP | EPOLLERR);
+	/*
+	 * Have it handle read data, too, so if there is a pending
+	 * error it will get handled.
+	 */
+	event.events |= EPOLLIN;
+    }
     if (event.events & (EPOLLIN | EPOLLHUP))
-	handle_selector_call(sel, fd, &sel->read_set,
-			     sel->fds[fd].handle_read);
+	handle_selector_call(sel, fd, &sel->read_set, fdc->handle_read);
     if (event.events & EPOLLOUT)
-	handle_selector_call(sel, fd, &sel->write_set,
-			     sel->fds[fd].handle_write);
-    if (event.events & (EPOLLERR | EPOLLPRI))
-	handle_selector_call(sel, fd, &sel->except_set,
-			     sel->fds[fd].handle_except);
+	handle_selector_call(sel, fd, &sel->write_set, fdc->handle_write);
+    if (event.events & (EPOLLPRI | EPOLLERR))
+	handle_selector_call(sel, fd, &sel->except_set, fdc->handle_except);
 
+ rearm:
     /* Rearm the event.  Remember it could have been deleted in the handler. */
-    if (sel->fds[fd].state)
-	sel_update_epoll(sel, fd, EPOLL_CTL_MOD);
+    if (fdc->state)
+	sel_update_fd(sel, fd, EPOLL_CTL_MOD);
     sel_fd_unlock(sel);
 
+    return rv;
+}
+
+int
+sel_setup_forked_process(struct selector_s *sel)
+{
+    int i;
+
+    /*
+     * More epoll stupidity.  In a forked process we must create a new
+     * epoll because the epoll state is shared between a parent and a
+     * child.  If it worked like it should, each epoll instance would
+     * be independent.  If you don't do this, disabling an fd in the
+     * child disables the parent, too, and vice versa.
+     */
+    close(sel->epollfd);
+    sel->epollfd = epoll_create(32768);
+    if (sel->epollfd == -1) {
+	return errno;
+    }
+
+    for (i = 0; i <= sel->maxfd; i++) {
+	volatile fd_control_t *fdc = &sel->fds[i];
+	if (fdc->state)
+	    sel_update_fd(sel, i, EPOLL_CTL_ADD);
+    }
+    return 0;
+}
+#else
+int
+sel_setup_forked_process(struct selector_s *sel)
+{
+    /* Nothing to do. */
     return 0;
 }
 #endif
 
+int
+sel_select_intr_sigmask(struct selector_s *sel,
+			sel_send_sig_cb send_sig,
+			long            thread_id,
+			void            *cb_data,
+			struct timeval  *timeout,
+			sigset_t        *sigmask)
+{
+    int             err, old_errno;
+    struct timeval  loc_timeout = { 0, 0 };
+    sel_wait_list_t wait_entry;
+    unsigned int    count;
+    struct timeval  end = { 0, 0 }, now;
+    int user_timeout = 0;
+
+    if (timeout) {
+	sel_get_monotonic_time(&now);
+	add_timeval(&end, &now, timeout);
+    }
+
+    sel_timer_lock(sel);
+    count = process_runners(sel);
+    process_timers(sel, &count, &loc_timeout);
+    if (timeout) {
+	if (cmp_timeval(&loc_timeout, timeout) >= 0) {
+	    loc_timeout = *timeout;
+	    user_timeout = 1;
+	}
+    }
+    add_sel_wait_list(sel, &wait_entry, send_sig, cb_data, thread_id);
+    sel_timer_unlock(sel);
+
+#ifdef HAVE_EPOLL_PWAIT
+    if (sel->epollfd >= 0)
+	err = process_fds_epoll(sel, &loc_timeout, sigmask);
+    else
+#endif
+	err = process_fds(sel, &loc_timeout, sigmask);
+
+    old_errno = errno;
+    if (!user_timeout && !err) {
+	/*
+	 * Only return a timeout if we waited on the user's timeout
+	 * Otherwise there is a timer to process.
+	 */
+	count++;
+	err = 0;
+    }
+
+    sel_timer_lock(sel);
+    remove_sel_wait_list(sel, &wait_entry);
+    sel_timer_unlock(sel);
+
+    if (timeout) {
+	sel_get_monotonic_time(&now);
+	diff_timeval(timeout, &end, &now);
+    }
+
+    if (err < 0) {
+	errno = old_errno;
+	return err;
+    }
+
+    return err + count;
+}
+
+int
+sel_select_intr(struct selector_s *sel,
+		sel_send_sig_cb send_sig,
+		long            thread_id,
+		void            *cb_data,
+		struct timeval  *timeout)
+{
+    return sel_select_intr_sigmask(sel, send_sig, thread_id, cb_data, timeout,
+				   NULL);
+}
+
 int
 sel_select(struct selector_s *sel,
 	   sel_send_sig_cb send_sig,
@@ -990,32 +1247,16 @@ sel_select(struct selector_s *sel,
 	   void            *cb_data,
 	   struct timeval  *timeout)
 {
-    int             err;
-    struct timeval  loc_timeout = { 0, 0 };
-    sel_wait_list_t wait_entry;
-
-    sel_timer_lock(sel);
-    process_runners(sel);
-    process_timers(sel, &loc_timeout);
-    if (timeout) {
-	if (cmp_timeval(&loc_timeout, timeout) >= 0)
-	    memcpy(&loc_timeout, timeout, sizeof(loc_timeout));
-    }
-    add_sel_wait_list(sel, &wait_entry, send_sig, cb_data, thread_id,
-		      &loc_timeout);
-    sel_timer_unlock(sel);
-
-#ifdef HAVE_EPOLL_PWAIT
-    if (sel->epollfd >= 0)
-	err = process_fds_epoll(sel, &loc_timeout);
-    else
-#endif
-	err = process_fds(sel, &loc_timeout);
-
-    sel_timer_lock(sel);
-    remove_sel_wait_list(sel, &wait_entry);
-    sel_timer_unlock(sel);
+    int err;
 
+    err = sel_select_intr_sigmask(sel, send_sig, thread_id, cb_data, timeout,
+				  NULL);
+    if (err < 0 && errno == EINTR)
+	/*
+	 * If we get an EINTR, we don't want to report a timeout.  Just
+	 * return that we did something.
+	 */
+	return 1;
     return err;
 }
 
@@ -1053,6 +1294,8 @@ sel_alloc_selector_thread(struct selector_s **new_selector, int wake_sig,
 {
     struct selector_s *sel;
     unsigned int i;
+    int rv;
+    sigset_t sigset;
 
     sel = malloc(sizeof(*sel));
     if (!sel)
@@ -1094,28 +1337,23 @@ sel_alloc_selector_thread(struct selector_s **new_selector, int wake_sig,
 	}
     }
 
+    sigemptyset(&sigset);
+    sigaddset(&sigset, wake_sig);
+    rv = sigprocmask(SIG_BLOCK, &sigset, NULL);
+    if (rv == -1) {
+	rv = errno;
+	if (sel->sel_lock_alloc) {
+	    sel->sel_lock_free(sel->fd_lock);
+		sel->sel_lock_free(sel->timer_lock);
+	}
+	free(sel);
+	return rv;
+    }
+
 #ifdef HAVE_EPOLL_PWAIT
     sel->epollfd = epoll_create(32768);
-    if (sel->epollfd == -1) {
+    if (sel->epollfd == -1)
 	syslog(LOG_ERR, "Unable to set up epoll, falling back to select: %m");
-    } else {
-	int rv;
-	sigset_t sigset;
-
-	sigemptyset(&sigset);
-	sigaddset(&sigset, wake_sig);
-	rv = sigprocmask(SIG_BLOCK, &sigset, NULL);
-	if (rv == -1) {
-	    rv = errno;
-	    close(sel->epollfd);
-	    if (sel->sel_lock_alloc) {
-		sel->sel_lock_free(sel->fd_lock);
-		sel->sel_lock_free(sel->timer_lock);
-	    }
-	    free(sel);
-	    return rv;
-	}
-    }
 #endif
 
     *new_selector = sel;
